Package: NonlinearDiD
Type: Package
Title: Staggered Difference-in-Differences with Nonlinear Outcomes
Version: 0.2.0
Description: Supports staggered difference-in-differences designs with
    nonlinear outcomes for both panel and repeated cross-section data.
    Implements estimators for staggered treatment adoption with binary,
    count, and other nonlinear outcomes, extending Callaway and Sant'Anna
    (2021) <doi:10.1016/j.jeconom.2020.12.001> to settings with nonlinear
    outcome models such as logit, probit, and Poisson. For panel data,
    units are followed over time and 'idname' identifies repeated
    observations. For repeated cross-section data, observations are
    independent within each time period; 'idname' is optional and may
    identify survey records or households, but the estimator does not
    require the same units to appear across periods. Repeated cross-section
    estimation includes pooled quasi-maximum likelihood approaches motivated
    by Wooldridge (2023) <doi:10.1093/ectj/utad016>, with optional weighting
    and clustered inference. Methods also draw on Roth and Sant'Anna (2023)
    <doi:10.3982/ECTA19402> and Sant'Anna and Zhao (2020)
    <doi:10.1016/j.jeconom.2020.06.003>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.0.0)
Imports: stats, utils, MASS, sandwich, lmtest, ggplot2
Suggests: did, dplyr, knitr, rmarkdown, testthat (>= 3.0.0), covr
Config/testthat/edition: 3
URL: https://github.com/causalfragility-lab/NonlinearDiD
BugReports: https://github.com/causalfragility-lab/NonlinearDiD/issues
NeedsCompilation: no
Authors@R: person(given = "Subir",
                  family = "Hait",
                  role = c("aut", "cre"),
                  email = "haitsubi@msu.edu",
                  comment = c(ORCID = "0009-0004-9871-9677"))
Packaged: 2026-05-20 14:10:49 UTC; Subir
Author: Subir Hait [aut, cre] (ORCID: <https://orcid.org/0009-0004-9871-9677>)
Maintainer: Subir Hait <haitsubi@msu.edu>
Repository: CRAN
Date/Publication: 2026-05-20 14:40:14 UTC
Built: R 4.5.2; ; 2026-05-25 01:32:56 UTC; unix

Package: NonlinearDiD
Type: Package
Title: Staggered Difference-in-Differences with Nonlinear Outcomes
Version: 0.2.0
Description: Supports staggered difference-in-differences designs with
    nonlinear outcomes for both panel and repeated cross-section data.
    Implements estimators for staggered treatment adoption with binary,
    count, and other nonlinear outcomes, extending Callaway and Sant'Anna
    (2021) <doi:10.1016/j.jeconom.2020.12.001> to settings with nonlinear
    outcome models such as logit, probit, and Poisson. For panel data,
    units are followed over time and 'idname' identifies repeated
    observations. For repeated cross-section data, observations are
    independent within each time period; 'idname' is optional and may
    identify survey records or households, but the estimator does not
    require the same units to appear across periods. Repeated cross-section
    estimation includes pooled quasi-maximum likelihood approaches motivated
    by Wooldridge (2023) <doi:10.1093/ectj/utad016>, with optional weighting
    and clustered inference. Methods also draw on Roth and Sant'Anna (2023)
    <doi:10.3982/ECTA19402> and Sant'Anna and Zhao (2020)
    <doi:10.1016/j.jeconom.2020.06.003>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.0.0)
Imports: stats, utils, MASS, sandwich, lmtest, ggplot2
Suggests: did, dplyr, knitr, rmarkdown, testthat (>= 3.0.0), covr
Config/testthat/edition: 3
URL: https://github.com/causalfragility-lab/NonlinearDiD
BugReports: https://github.com/causalfragility-lab/NonlinearDiD/issues
NeedsCompilation: no
Authors@R: person(given = "Subir",
                  family = "Hait",
                  role = c("aut", "cre"),
                  email = "haitsubi@msu.edu",
                  comment = c(ORCID = "0009-0004-9871-9677"))
Packaged: 2026-05-20 14:10:49 UTC; Subir
Author: Subir Hait [aut, cre] (ORCID: <https://orcid.org/0009-0004-9871-9677>)
Maintainer: Subir Hait <haitsubi@msu.edu>
Repository: CRAN
Date/Publication: 2026-05-20 14:40:14 UTC
Built: R 4.5.2; ; 2026-05-21 23:30:34 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,nonlinear_aggte)
S3method(plot,nonlinear_attgt)
S3method(print,binary_did_dr)
S3method(print,binary_did_logit)
S3method(print,binary_did_probit)
S3method(print,count_did_poisson)
S3method(print,nonlinear_aggte)
S3method(print,nonlinear_attgt)
S3method(print,nonlinear_pretest)
S3method(print,odds_ratio_did)
S3method(summary,nonlinear_aggte)
S3method(summary,nonlinear_attgt)
export(binary_did_dr)
export(binary_did_logit)
export(binary_did_probit)
export(count_did_poisson)
export(nonlinear_aggte)
export(nonlinear_attgt)
export(nonlinear_bounds)
export(nonlinear_pretest)
export(odds_ratio_did)
export(sim_binary_panel)
export(sim_binary_rcs)
export(sim_count_panel)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)

# NonlinearDiD

**Staggered Difference-in-Differences with Nonlinear Outcomes — panel and repeated cross-section data**

[![R-CMD-check](https://img.shields.io/badge/R--CMD--check-passing-brightgreen)](https://github.com/causalfragility-lab/NonlinearDiD)
[![CRAN status](https://www.r-pkg.org/badges/version/NonlinearDiD)](https://CRAN.R-project.org/package=NonlinearDiD)
[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)

## The Problem

The Callaway & Sant'Anna (2021) framework for staggered DiD is hugely influential — but it assumes continuous outcomes with linear parallel trends.

For binary outcomes (employed/not, hospitalized/not, defaulted/not), this creates fundamental problems:

| Problem | Why it matters |
|---|---|
| **Scale sensitivity** | Parallel trends in P(Y=1) ≠ parallel trends in log-odds. Pre-trends can appear flat or steep depending on the scale. |
| **Jensen's inequality** | Treatment effects on the probability scale mix the "real" effect with curvature of the CDF. |
| **Heterogeneous baseline rates** | Units with different baseline probabilities will show "spurious" violations of parallel trends even under no treatment effect. |

`NonlinearDiD` extends the CS2021 framework to properly handle logit, probit, Poisson, and negative binomial outcome models — for both **panel** and **repeated cross-section** data.

## What's New in 0.2.0

- **Repeated cross-section support** — different individuals each period (e.g. BRFSS, NHIS, CPS supplements). Set `data_type = "repeated_cross_section"`; `idname` becomes optional.
- **Sampling weights** — pass `weightsname = "wt"` and the weight is threaded through the outcome regression, propensity score, and pooled QMLE.
- **Clustered inference** — pass `cluster_var = "state"` for `sandwich::vcovCL()` analytical SEs and cluster-resampling bootstrap.
- **No more compilation** — the Rcpp helpers from 0.1.0 are now pure R, so installation is one step on every platform.
- **All v0.1.0 functions and arguments preserved** — existing scripts using named arguments continue to work unchanged.

## Installation

```r
# CRAN
install.packages("NonlinearDiD")

# GitHub (development version)
remotes::install_github("causalfragility-lab/NonlinearDiD")
```

## Quick Start: Panel Data

```r
library(NonlinearDiD)

# 1. Simulate staggered binary panel data
dat <- sim_binary_panel(n = 500, nperiods = 8, n_cohorts = 3,
                        prop_treated = 0.5, true_att = 0.25, seed = 42)

# 2. Estimate ATT(g,t) with logistic outcome model
res <- nonlinear_attgt(
  data          = dat,
  yname         = "y",
  tname         = "period",
  idname        = "id",
  gname         = "g",
  xformla       = ~ x1 + x2,
  outcome_model = "logit",
  estimand      = "att",
  control_group = "nevertreated",
  doubly_robust = TRUE
)

# 3. Aggregate into event-study
agg <- nonlinear_aggte(res, type = "dynamic")
plot(agg)

# 4. Pre-treatment parallel trends test
nonlinear_pretest(res)
```

## Quick Start: Repeated Cross-Section Data

```r
library(NonlinearDiD)

# 1. Simulate repeated cross-section binary data
rcs <- sim_binary_rcs(n_per_period = 500, nperiods = 8,
                      prop_treated = 0.5, true_att = 0.3, seed = 7)

# 2. Estimate ATT(g,t) — note the new data_type argument
res <- nonlinear_attgt(
  data          = rcs,
  yname         = "y",
  tname         = "period",
  gname         = "g",
  outcome_model = "logit",
  estimand      = "ape",
  data_type     = "repeated_cross_section",
  control_group = "notyetreated"
)

plot(nonlinear_aggte(res, type = "dynamic"))
```

## Survey-Weighted Real-World Example

Repeated cross-section survey data (e.g. CPS Food Security Supplement) with sampling weights and state-level clustering:

```r
res <- nonlinear_attgt(
  data          = snap_data,
  yname         = "food_insecure",
  tname         = "year",
  gname         = "policy_end_year",
  idname        = "household_id",    # optional — used only as a record ID
  data_type     = "repeated_cross_section",
  outcome_model = "logit",
  estimand      = "ape",
  weightsname   = "survey_weight",
  cluster_var   = "state",
  control_group = "notyetreated"
)

summary(res)
nonlinear_aggte(res, type = "dynamic")
```

## Key Functions

| Function | Description |
|---|---|
| `nonlinear_attgt()` | Main engine: estimates ATT(g,t) for all cohort × time cells. Panel and repeated cross-section. |
| `nonlinear_aggte()` | Aggregates ATT(g,t) into event-study, group, calendar, or overall ATT |
| `nonlinear_pretest()` | Tests pre-treatment parallel trends (joint + individual + HonestDiD) |
| `binary_did_logit()` | Simple 2×2 DiD with logistic outcome |
| `binary_did_probit()` | Simple 2×2 DiD with probit outcome |
| `binary_did_dr()` | Doubly-robust binary DiD |
| `count_did_poisson()` | Poisson QMLE DiD for count outcomes (Wooldridge 2023) |
| `odds_ratio_did()` | Odds-ratio DiD estimator |
| `nonlinear_bounds()` | Nonparametric Manski / PT bounds |
| `sim_binary_panel()` | Simulate binary panel data for testing |
| `sim_binary_rcs()` | Simulate binary repeated cross-section data **(new in 0.2.0)** |
| `sim_count_panel()` | Simulate count panel data for testing |

## Estimands

| Estimand | Scale | When to use |
|---|---|---|
| `"att"` | Link scale (log-odds / probit index / log-count) | Compare with linear DiD coefficient |
| `"ape"` | Probability scale | What practitioners usually report |
| `"odds_ratio"` | Multiplicative | Scale-free; natural for 2×2 tables |

## Panel vs Repeated Cross-Section

`NonlinearDiD` supports staggered difference-in-differences designs with nonlinear outcomes for both panel and repeated cross-section data.

- **Panel data** (`data_type = "panel"`, default): units are followed over time and `idname` identifies repeated observations. Estimation uses within-unit outcome changes following Callaway & Sant'Anna (2021).

- **Repeated cross-section data** (`data_type = "repeated_cross_section"`): observations are independent within each time period. `idname` is optional and may identify survey records or households, but the estimator does not require the same units to appear across periods. Estimation uses pooled quasi-maximum likelihood approaches motivated by Wooldridge (2023), with an optional IPW-augmented doubly-robust variant.

## Outcome Models

| `outcome_model` | Parallel Trends Assumption | Outcome Type |
|---|---|---|
| `"logit"` | Parallel in log-odds | Binary (0/1) |
| `"probit"` | Parallel in probit index | Binary (0/1) |
| `"poisson"` | Parallel in log-count | Count (≥ 0) |
| `"negbin"` | Parallel in log-count | Overdispersed count |
| `"linear"` | Parallel in mean (LPM) | Continuous / binary |

## References

- Callaway, B., & Sant'Anna, P. H. C. (2021). Difference-in-differences with multiple time periods. *Journal of Econometrics*, 225(2), 200–230. <https://doi.org/10.1016/j.jeconom.2020.12.001>
- Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive to functional form? *Econometrica*, 91(2), 737–747. <https://doi.org/10.3982/ECTA19402>
- Wooldridge, J. M. (2023). Simple approaches to nonlinear difference-in-differences with panel data. *The Econometrics Journal*, 26(3), C31–C66. <https://doi.org/10.1093/ectj/utad016>
- Sant'Anna, P. H. C., & Zhao, J. (2020). Doubly robust difference-in-differences estimators. *Journal of Econometrics*, 219(1), 101–122. <https://doi.org/10.1016/j.jeconom.2020.06.003>
- Manski, C. F. (1990). Nonparametric bounds on treatment effects. *American Economic Review*, 80(2), 319–323.

## Contributing

This package addresses an active research frontier. Contributions, bug reports, and methodological suggestions are welcome — please open an issue or pull request on GitHub.

## License

MIT © 2026 Subir Hait

# Run this ONCE before devtools::document() if upgrading from a prior version.
# Removes stale auto-generated .Rd files so the fresh correct ones are written.
# Usage (from RStudio with package root as working dir):
#   source("inst/cleanup_stale_rd.R")

man_dir <- "man"
if (!dir.exists(man_dir)) {
  message("man/ directory not found — nothing to clean.")
} else {
  old_rds <- list.files(man_dir, pattern = "\\.Rd$", full.names = TRUE)
  if (length(old_rds) == 0) {
    message("No stale .Rd files found.")
  } else {
    message("Removing ", length(old_rds), " stale .Rd file(s):")
    message(paste(" -", basename(old_rds), collapse = "\n"))
    invisible(file.remove(old_rds))
    message("\nDone. Now run devtools::document() then devtools::install().")
  }
}

#' @title Inference for Nonlinear DiD
#' @description Internal functions for bootstrap and delta-method standard
#'   errors in nonlinear staggered DiD, for both panel and repeated
#'   cross-section designs, with optional clustering and sampling weights.
#' @keywords internal

# ============================================================
# Multiplier / Empirical Bootstrap
# ============================================================

.bootstrap_inference <- function(attgt_df, data, yname, tname, idname, gname,
                                 xformla, outcome_model, estimand, control_group,
                                 doubly_robust, nboot, boot_type, alpha,
                                 anticipation, parallel, pl_cores,
                                 data_type = "panel", cluster_var = NULL) {

  tlist <- sort(unique(data[[tname]]))
  glist <- sort(setdiff(unique(data[[gname]]), 0))

  # Decide what gets resampled.
  # 1. cluster_var provided  -> cluster bootstrap
  # 2. data_type panel       -> unit-id bootstrap
  # 3. data_type RCS         -> row bootstrap
  if (!is.null(cluster_var)) {
    cluster_ids <- unique(data[[cluster_var]])
    boot_unit_col <- cluster_var
    n_units <- length(cluster_ids)
    ids     <- cluster_ids
  } else if (data_type == "panel") {
    ids     <- unique(data[[idname]])
    n_units <- length(ids)
    boot_unit_col <- idname
  } else {
    ids     <- seq_len(nrow(data))
    n_units <- nrow(data)
    boot_unit_col <- NULL  # row-level
  }

  if (parallel && requireNamespace("parallel", quietly = TRUE)) {
    cl <- parallel::makeCluster(pl_cores)
    on.exit(parallel::stopCluster(cl), add = TRUE)
    boot_draws <- parallel::parSapply(cl, seq_len(nboot), function(b)
      .single_boot(b, ids, n_units, data, yname, tname, idname, gname,
                   xformla, outcome_model, estimand, control_group,
                   doubly_robust, boot_type, anticipation, glist, tlist,
                   data_type, boot_unit_col))
  } else {
    boot_draws <- matrix(NA_real_, nrow = nrow(attgt_df), ncol = nboot)
    for (b in seq_len(nboot)) {
      bv <- .single_boot(b, ids, n_units, data, yname, tname, idname, gname,
                         xformla, outcome_model, estimand, control_group,
                         doubly_robust, boot_type, anticipation, glist, tlist,
                         data_type, boot_unit_col)
      boot_draws[, b] <- bv
    }
  }

  attgt_df$se    <- apply(boot_draws, 1, function(x) stats::sd(x, na.rm = TRUE))
  attgt_df$ci_lo <- attgt_df$att - stats::qnorm(1 - alpha / 2) * attgt_df$se
  attgt_df$ci_hi <- attgt_df$att + stats::qnorm(1 - alpha / 2) * attgt_df$se

  # Simultaneous CIs from bootstrap quantile
  attgt_df$ci_lo_simult <- NA_real_
  attgt_df$ci_hi_simult <- NA_real_
  for (i in seq_len(nrow(attgt_df))) {
    q <- stats::quantile(abs(boot_draws[i, ] - attgt_df$att[i]),
                          1 - alpha, na.rm = TRUE)
    attgt_df$ci_lo_simult[i] <- attgt_df$att[i] - q
    attgt_df$ci_hi_simult[i] <- attgt_df$att[i] + q
  }

  attgt_df
}


.single_boot <- function(b, ids, n_units, data, yname, tname, idname, gname,
                         xformla, outcome_model, estimand, control_group,
                         doubly_robust, boot_type, anticipation, glist, tlist,
                         data_type = "panel", boot_unit_col = NULL) {

  # Build boot_data:
  # - boot_unit_col not NULL: cluster/unit bootstrap on that column
  # - boot_unit_col NULL    : row-level bootstrap (RCS no cluster)
  if (!is.null(boot_unit_col)) {
    if (boot_type == "multiplier") {
      weights_vec <- stats::rexp(n_units)
      weights_vec <- weights_vec / mean(weights_vec)
      id_weight_map <- stats::setNames(weights_vec, ids)
      boot_data <- data
      boot_data$.boot_wt <- id_weight_map[as.character(boot_data[[boot_unit_col]])]
      # Fold into sampling weight
      boot_data$.sw <- boot_data$.sw * boot_data$.boot_wt
    } else {
      sampled_ids <- sample(ids, n_units, replace = TRUE)
      boot_data <- do.call(rbind, lapply(seq_along(sampled_ids), function(k) {
        sub <- data[data[[boot_unit_col]] == sampled_ids[k], , drop = FALSE]
        # Re-label the unit to keep rows uniquely identifiable in panel mode
        if (data_type == "panel" && !is.null(idname)) {
          sub[[idname]] <- paste0("boot_", k, "_", sub[[idname]])
        }
        sub
      }))
    }
  } else {
    # Row-level (RCS, no clustering)
    if (boot_type == "multiplier") {
      row_wts <- stats::rexp(n_units)
      row_wts <- row_wts / mean(row_wts)
      boot_data <- data
      boot_data$.sw <- boot_data$.sw * row_wts
    } else {
      # Resample rows within each (group x period) cell
      cells <- split(seq_len(nrow(data)), list(data[[gname]], data[[tname]]))
      new_rows <- unlist(lapply(cells, function(idx) {
        if (length(idx) == 0) return(integer(0))
        sample(idx, length(idx), replace = TRUE)
      }))
      boot_data <- data[new_rows, , drop = FALSE]
      rownames(boot_data) <- NULL
    }
  }

  atts <- numeric(length(glist) * length(tlist))
  idx  <- 0L

  for (g in glist) {
    for (t in tlist) {
      idx <- idx + 1L
      pre_period <- g - 1L - anticipation
      res <- tryCatch(
        .compute_attgt_single(
          data = boot_data, yname = yname, tname = tname,
          idname = idname, gname = gname, xformla = xformla,
          g = g, t = t, base_period = pre_period,
          outcome_model = outcome_model, estimand = estimand,
          control_group = control_group, doubly_robust = doubly_robust,
          data_type = data_type
        ),
        error = function(e) list(att = NA_real_)
      )
      atts[idx] <- if (is.null(res$att)) NA_real_ else res$att
    }
  }
  atts
}


# ============================================================
# Analytical SEs (no bootstrap)
# ============================================================

.sandwich_se <- function(attgt_df, data, yname, tname, idname, gname,
                         xformla, outcome_model, alpha,
                         data_type = "panel", cluster_var = NULL) {

  if (data_type == "repeated_cross_section") {
    return(.sandwich_se_rcs(
      attgt_df = attgt_df, data = data, yname = yname, tname = tname,
      gname = gname, xformla = xformla, outcome_model = outcome_model,
      alpha = alpha, cluster_var = cluster_var
    ))
  }

  # ---- Panel path (v0.1.0 IF-based SEs; clustering left to bootstrap) ----
  ids <- unique(data[[idname]])
  n   <- length(ids)

  for (i in seq_len(nrow(attgt_df))) {
    g <- attgt_df$group[i]
    t <- attgt_df$time[i]
    if (is.na(attgt_df$att[i])) next

    se_hat <- tryCatch({
      .if_se(data = data, yname = yname, tname = tname, idname = idname,
             gname = gname, xformla = xformla, g = g, t = t,
             outcome_model = outcome_model, n = n)
    }, error = function(e) NA_real_)

    attgt_df$se[i] <- se_hat
  }

  if (!is.null(cluster_var)) {
    # Panel + clustering with analytical SEs is not yet implemented.
    # Inflate the IF-based SE by a conservative DEFF-style adjustment
    # based on average cluster size, so users still get a SE estimate.
    # Recommend bootstrap with cluster_var for proper clustered SEs.
    n_per_cluster <- as.numeric(table(data[[cluster_var]]))
    rho           <- 0.05  # mild intra-cluster correlation assumption
    avg_n         <- mean(n_per_cluster)
    deff          <- 1 + (avg_n - 1) * rho
    attgt_df$se   <- attgt_df$se * sqrt(deff)
    message("Note: panel + cluster_var analytical SEs use a DEFF approximation. ",
            "For full clustered inference, set boot = TRUE.")
  }

  attgt_df$ci_lo <- attgt_df$att - stats::qnorm(1 - alpha / 2) * attgt_df$se
  attgt_df$ci_hi <- attgt_df$att + stats::qnorm(1 - alpha / 2) * attgt_df$se
  attgt_df
}


# ============================================================
# RCS analytical SEs: HC1 or vcovCL on pooled GLM
# ============================================================

.sandwich_se_rcs <- function(attgt_df, data, yname, tname, gname,
                              xformla, outcome_model, alpha,
                              cluster_var = NULL) {

  for (i in seq_len(nrow(attgt_df))) {
    g <- attgt_df$group[i]
    t <- attgt_df$time[i]
    if (is.na(attgt_df$att[i])) next

    se_hat <- tryCatch({
      .rcs_if_se(
        data = data, yname = yname, tname = tname,
        gname = gname, xformla = xformla,
        outcome_model = outcome_model,
        g = g, t = t, cluster_var = cluster_var
      )
    }, error = function(e) NA_real_)

    attgt_df$se[i] <- se_hat
  }

  attgt_df$ci_lo <- attgt_df$att - stats::qnorm(1 - alpha / 2) * attgt_df$se
  attgt_df$ci_hi <- attgt_df$att + stats::qnorm(1 - alpha / 2) * attgt_df$se
  attgt_df
}


# Extract SE for the D:post interaction from a pooled GLM fit on the
# (g, t) cell subset. Uses sandwich::vcovCL if cluster_var is supplied,
# otherwise sandwich::vcovHC (HC1).
.rcs_if_se <- function(data, yname, tname, gname, xformla, outcome_model,
                       g, t, cluster_var = NULL) {

  pre_period <- g - 1L

  sub <- data[(data[[gname]] == g | data[[gname]] == 0) &
                data[[tname]] %in% c(pre_period, t), , drop = FALSE]

  if (nrow(sub) < 8) return(NA_real_)

  sub$D    <- as.integer(sub[[gname]] == g)
  sub$post <- as.integer(sub[[tname]] == t)
  sub$did  <- sub$D * sub$post

  fam   <- .outcome_family(outcome_model)
  xvars <- labels(stats::terms(xformla))
  fmla  <- .rcs_formula(yname, xvars)
  environment(fmla) <- environment()

  sw <- if (".sw" %in% names(sub)) sub$.sw else rep(1.0, nrow(sub))

  fit <- suppressWarnings(stats::glm(fmla, data = sub, family = fam,
                                      weights = sw))

  if (!("did" %in% names(stats::coef(fit)))) return(NA_real_)

  vcov_hc <- tryCatch({
    if (!is.null(cluster_var) && cluster_var %in% names(sub)) {
      sandwich::vcovCL(fit, cluster = sub[[cluster_var]], type = "HC1")
    } else {
      sandwich::vcovHC(fit, type = "HC1")
    }
  }, error = function(e) NULL)

  if (is.null(vcov_hc) || !("did" %in% rownames(vcov_hc))) return(NA_real_)
  sqrt(vcov_hc["did", "did"])
}


# ============================================================
# Panel influence-function SE (v0.1.0, unchanged behaviour)
# ============================================================

.if_se <- function(data, yname, tname, idname, gname, xformla, g, t,
                   outcome_model, n) {

  pre_period <- g - 1L

  relevant_ids <- data[[idname]][data[[gname]] == g | data[[gname]] == 0]
  sub <- data[data[[idname]] %in% relevant_ids &
                data[[tname]] %in% c(pre_period, t), , drop = FALSE]

  if (nrow(sub) < 4) return(NA_real_)

  sub$D    <- as.integer(sub[[gname]] == g)
  sub_wide <- .make_wide(sub, idname, tname, yname, pre_period, t)
  sub_wide$D <- as.integer(sub_wide[["D_t1"]])

  if (nrow(sub_wide) < 4) return(NA_real_)

  Y0  <- sub_wide[[paste0(yname, "_t0")]]
  Y1  <- sub_wide[[paste0(yname, "_t1")]]
  D   <- sub_wide$D
  n_g <- nrow(sub_wide)

  ps_formula <- stats::update(xformla, D ~ .)
  ps_fit     <- suppressWarnings(
    stats::glm(ps_formula, data = sub_wide, family = stats::binomial()))
  pscore     <- pmin(pmax(stats::predict(ps_fit, type = "response"), 1e-6), 1 - 1e-6)

  controls_df    <- sub_wide[D == 0, , drop = FALSE]
  controls_df$DY <- Y1[D == 0] - Y0[D == 0]
  or_formula     <- stats::update(xformla, DY ~ .)
  or_fit         <- suppressWarnings(stats::lm(or_formula, data = controls_df))
  mu_hat         <- stats::predict(or_fit, newdata = sub_wide)

  DeltaY <- Y1 - Y0
  p_D    <- mean(D)
  w_t    <- D / p_D
  w_c    <- (1 - D) * pscore / (1 - pscore) / p_D

  psi_att <- w_t * (DeltaY - mu_hat) - w_c * (DeltaY - mu_hat)

  se_hat <- sqrt(stats::var(psi_att, na.rm = TRUE) / n_g) * sqrt(n_g / n)
  se_hat
}

# Helper: build a formula safely from a base formula + covariates
.make_formula <- function(lhs_str, base_rhs_str, xformla) {
  xvars <- labels(stats::terms(xformla))
  if (length(xvars) > 0 && !(length(xvars) == 1 && xvars == "")) {
    rhs <- paste(c(base_rhs_str, xvars), collapse = " + ")
  } else {
    rhs <- base_rhs_str
  }
  stats::as.formula(paste(lhs_str, "~", rhs))
}

#' @title Binary Outcome DiD: Logit Estimator
#'
#' @description
#' Estimates a 2x2 difference-in-differences model with a binary outcome using
#' logistic regression on the log-odds scale, reporting both the log-odds DiD
#' coefficient and the average partial effect (APE) on the probability scale.
#'
#' @param data A data frame (long format).
#' @param yname Character. Binary outcome variable name.
#' @param tname Character. Time period variable name.
#' @param idname Character. Unit ID variable name.
#' @param treat_period Numeric. The treatment (post) period.
#' @param control_period Numeric. The pre-treatment baseline period.
#' @param dname Character. Treatment indicator variable name (optional).
#' @param gname Character. Cohort variable name (optional).
#' @param xformla One-sided formula for covariates. Default \code{~1}.
#' @param se_type Character. SE type: \code{"robust"} (default),
#'   \code{"cluster"}, or \code{"analytical"}.
#' @param cluster_var Character. Clustering variable (if \code{se_type = "cluster"}).
#'
#' @return A list of class \code{binary_did_logit}.
#' @examples
#' dat <- sim_binary_panel(n = 500, nperiods = 4, prop_treated = 0.5)
#' dat2 <- dat[dat$period %in% c(2, 3), ]
#' res <- binary_did_logit(dat2, yname = "y", tname = "period",
#'                          idname = "id", treat_period = 3,
#'                          control_period = 2, gname = "g")
#' print(res)
#' @export
binary_did_logit <- function(
    data, yname, tname, idname, treat_period, control_period,
    dname = NULL, gname = NULL, xformla = ~1,
    se_type = c("robust", "cluster", "analytical"), cluster_var = NULL
) {
  se_type <- match.arg(se_type)
  .binary_did_impl(data, yname, tname, idname, treat_period, control_period,
                    dname, gname, xformla, se_type, cluster_var, link = "logit")
}

#' @title Binary Outcome DiD: Probit Estimator
#'
#' @description
#' Estimates 2x2 DiD with binary outcome using probit regression.
#' Parallel trends assumed on the probit (inverse-normal) scale.
#'
#' @inheritParams binary_did_logit
#' @return A list of class \code{binary_did_probit}.
#' @examples
#' dat <- sim_binary_panel(n = 500, nperiods = 4, prop_treated = 0.5)
#' dat2 <- dat[dat$period %in% c(2, 3), ]
#' res <- binary_did_probit(dat2, "y", "period", "id", 3, 2, gname = "g")
#' print(res)
#' @export
binary_did_probit <- function(
    data, yname, tname, idname, treat_period, control_period,
    dname = NULL, gname = NULL, xformla = ~1,
    se_type = c("robust", "cluster", "analytical"), cluster_var = NULL
) {
  se_type <- match.arg(se_type)
  .binary_did_impl(data, yname, tname, idname, treat_period, control_period,
                    dname, gname, xformla, se_type, cluster_var, link = "probit")
}

#' @title Doubly-Robust Binary DiD
#'
#' @description
#' Doubly-robust estimator for binary outcomes combining a nonlinear outcome
#' regression model with inverse probability weighting via propensity score.
#' Consistent if EITHER the outcome model OR the propensity score is correctly
#' specified.
#'
#' @inheritParams binary_did_logit
#' @param outcome_model Character. \code{"logit"} (default) or \code{"probit"}.
#' @return A list of class \code{binary_did_dr}.
#' @examples
#' dat <- sim_binary_panel(n = 500, nperiods = 4, prop_treated = 0.5)
#' dat2 <- dat[dat$period %in% c(2, 3), ]
#' res <- binary_did_dr(dat2, "y", "period", "id", 3, 2, gname = "g",
#'                       outcome_model = "logit")
#' print(res)
#' @export
binary_did_dr <- function(
    data, yname, tname, idname, treat_period, control_period,
    dname = NULL, gname = NULL, xformla = ~1,
    outcome_model = c("logit", "probit"),
    se_type = c("robust", "cluster", "analytical"), cluster_var = NULL
) {
  outcome_model <- match.arg(outcome_model)
  se_type       <- match.arg(se_type)

  sub <- data[data[[tname]] %in% c(control_period, treat_period), , drop = FALSE]
  sub$D <- if (!is.null(gname)) as.integer(sub[[gname]] != 0) else
            if (!is.null(dname)) as.integer(sub[[dname]]) else
            stop("Must provide either 'gname' or 'dname'.")

  sub_wide <- .make_wide(sub, idname, tname, yname, control_period, treat_period)
  sub_wide$D <- as.integer(sub_wide$D_t1)

  Y0 <- sub_wide[[paste0(yname, "_t0")]]
  Y1 <- sub_wide[[paste0(yname, "_t1")]]
  D  <- sub_wide$D

  ps_formula <- stats::update(xformla, D ~ .)
  ps_fit     <- suppressWarnings(
    stats::glm(ps_formula, data = sub_wide, family = stats::binomial()))
  pscore <- pmin(pmax(stats::predict(ps_fit, type = "response"), 1e-6), 1 - 1e-6)

  att_val <- .dr_att_linear(Y0, Y1, D, pscore, rep(1, length(D)), sub_wide, xformla, outcome_model)

  DeltaY <- Y1 - Y0
  p_D    <- mean(D)
  w_t    <- D / p_D
  w_c    <- (1 - D) * pscore / (1 - pscore) / p_D

  ctrl_df     <- sub_wide[D == 0, , drop = FALSE]
  ctrl_df$DY  <- DeltaY[D == 0]
  or_fit      <- suppressWarnings(stats::lm(.make_formula("DY", "1", xformla),
                                             data = ctrl_df))
  mu_hat <- stats::predict(or_fit, newdata = sub_wide)
  psi    <- w_t * (DeltaY - mu_hat) - w_c * (DeltaY - mu_hat)
  se_val <- stats::sd(psi) / sqrt(length(psi))

  out <- list(
    att = att_val, se = se_val,
    ci_lo = att_val - 1.96 * se_val, ci_hi = att_val + 1.96 * se_val,
    tstat = att_val / se_val,
    pval  = 2 * stats::pnorm(-abs(att_val / se_val)),
    n = nrow(sub_wide), n_treat = sum(D), n_cont = sum(1 - D),
    outcome_model = outcome_model
  )
  class(out) <- "binary_did_dr"
  out
}

#' @title Count Outcome DiD: Poisson Estimator
#'
#' @description
#' Estimates DiD for count outcomes using a Poisson quasi-maximum likelihood
#' (QMLE) estimator with a log-linear parallel trends assumption.
#' The treatment effect is a multiplicative rate ratio.
#'
#' @inheritParams binary_did_logit
#' @param offset Character. Name of offset variable. Default \code{NULL}.
#' @return A list of class \code{count_did_poisson}.
#' @examples
#' dat <- sim_count_panel(n = 400, nperiods = 6, prop_treated = 0.4)
#' dat2 <- dat[dat$period %in% c(2, 4), ]
#' res <- count_did_poisson(dat2, "y", "period", "id", 4, 2, gname = "g")
#' print(res)
#' @export
count_did_poisson <- function(
    data, yname, tname, idname, treat_period, control_period,
    dname = NULL, gname = NULL, xformla = ~1, offset = NULL,
    se_type = c("robust", "cluster", "analytical"), cluster_var = NULL
) {
  se_type <- match.arg(se_type)

  sub <- data[data[[tname]] %in% c(control_period, treat_period), , drop = FALSE]
  sub$D <- if (!is.null(gname)) as.integer(sub[[gname]] != 0) else
            if (!is.null(dname)) as.integer(sub[[dname]]) else
            stop("Must provide 'gname' or 'dname'.")

  sub$post <- as.integer(sub[[tname]] == treat_period)
  sub$did  <- sub$D * sub$post

  fit_formula <- .make_formula(yname, "D + post + did", xformla)

  if (!is.null(offset)) {
    fit <- suppressWarnings(
      stats::glm(fit_formula, data = sub, family = stats::poisson(link = "log"),
                 offset = log(sub[[offset]] + 1))
    )
  } else {
    fit <- suppressWarnings(
      stats::glm(fit_formula, data = sub, family = stats::quasipoisson(link = "log"))
    )
  }

  vcov_mat <- if (se_type == "robust") sandwich::vcovHC(fit, type = "HC1") else stats::vcov(fit)
  coef_tab <- lmtest::coeftest(fit, vcov = vcov_mat)
  did_coef <- stats::coef(fit)["did"]
  did_se   <- sqrt(vcov_mat["did", "did"])
  rate_ratio <- exp(did_coef)
  mean_y0    <- mean(sub[[yname]][sub$D == 1 & sub$post == 0], na.rm = TRUE)

  out <- list(
    att_log_rr = did_coef, se_log_rr = did_se,
    rate_ratio = rate_ratio,
    att_ape    = mean_y0 * (rate_ratio - 1),
    ci_lo_log  = did_coef - 1.96 * did_se,
    ci_hi_log  = did_coef + 1.96 * did_se,
    ci_lo_rr   = exp(did_coef - 1.96 * did_se),
    ci_hi_rr   = exp(did_coef + 1.96 * did_se),
    tstat      = did_coef / did_se,
    pval       = 2 * stats::pnorm(-abs(did_coef / did_se)),
    fit        = fit, coef_table = coef_tab
  )
  class(out) <- "count_did_poisson"
  out
}

#' @title Odds-Ratio DiD for Binary Outcomes
#'
#' @description
#' Estimates the odds-ratio difference-in-differences (OR-DiD) for binary
#' outcomes. OR-DiD equals 1 under no treatment effect and is invariant to
#' which group is labelled treatment.
#'
#' @inheritParams binary_did_logit
#' @return A list of class \code{odds_ratio_did}.
#' @examples
#' dat <- sim_binary_panel(n = 500, nperiods = 4, prop_treated = 0.5)
#' dat2 <- dat[dat$period %in% c(2, 3), ]
#' res <- odds_ratio_did(dat2, "y", "period", "id", 3, 2, gname = "g")
#' print(res)
#' @export
odds_ratio_did <- function(
    data, yname, tname, idname, treat_period, control_period,
    dname = NULL, gname = NULL, xformla = ~1
) {
  sub <- data[data[[tname]] %in% c(control_period, treat_period), , drop = FALSE]
  sub$D <- if (!is.null(gname)) as.integer(sub[[gname]] != 0) else
            if (!is.null(dname)) as.integer(sub[[dname]]) else
            stop("Must provide 'gname' or 'dname'.")

  sub$post <- as.integer(sub[[tname]] == treat_period)
  sub$did  <- sub$D * sub$post

  fit_formula <- .make_formula(yname, "D + post + did", xformla)
  fit <- suppressWarnings(
    stats::glm(fit_formula, data = sub, family = stats::binomial(link = "logit"))
  )

  vcov_rob   <- sandwich::vcovHC(fit, type = "HC1")
  log_or_did <- stats::coef(fit)["did"]
  se_log_or  <- sqrt(vcov_rob["did", "did"])
  or_did     <- exp(log_or_did)

  out <- list(
    log_or_did = log_or_did, se_log_or = se_log_or,
    or_did     = or_did,
    ci_lo_log  = log_or_did - 1.96 * se_log_or,
    ci_hi_log  = log_or_did + 1.96 * se_log_or,
    ci_lo_or   = exp(log_or_did - 1.96 * se_log_or),
    ci_hi_or   = exp(log_or_did + 1.96 * se_log_or),
    tstat      = log_or_did / se_log_or,
    pval       = 2 * stats::pnorm(-abs(log_or_did / se_log_or)),
    fit        = fit,
    interpretation = paste0(
      "OR-DiD = ", round(or_did, 3),
      " [95% CI: ", round(exp(log_or_did - 1.96 * se_log_or), 3),
      ", ", round(exp(log_or_did + 1.96 * se_log_or), 3), "]"
    )
  )
  class(out) <- "odds_ratio_did"
  out
}

# ============================================================
# Internal implementation
# ============================================================

.binary_did_impl <- function(data, yname, tname, idname, treat_period,
                               control_period, dname, gname, xformla,
                               se_type, cluster_var, link) {

  sub <- data[data[[tname]] %in% c(control_period, treat_period), , drop = FALSE]
  sub$D <- if (!is.null(gname)) as.integer(sub[[gname]] != 0) else
            if (!is.null(dname)) as.integer(sub[[dname]]) else
            stop("Must provide either 'gname' or 'dname'.")

  sub$post <- as.integer(sub[[tname]] == treat_period)
  sub$did  <- sub$D * sub$post

  fit_formula <- .make_formula(yname, "D + post + did", xformla)
  fit <- suppressWarnings(
    stats::glm(fit_formula, data = sub, family = stats::binomial(link = link))
  )

  vcov_mat <- if (se_type == "robust") {
    sandwich::vcovHC(fit, type = "HC1")
  } else if (se_type == "cluster" && !is.null(cluster_var)) {
    sandwich::vcovCL(fit, cluster = sub[[cluster_var]])
  } else {
    stats::vcov(fit)
  }

  coef_tab <- lmtest::coeftest(fit, vcov = vcov_mat)
  did_coef <- stats::coef(fit)["did"]
  did_se   <- sqrt(vcov_mat["did", "did"])

  link_fn  <- if (link == "logit") stats::plogis else stats::pnorm
  dlink_fn <- if (link == "logit") {
    function(x) stats::plogis(x) * (1 - stats::plogis(x))
  } else {
    stats::dnorm
  }
  lp       <- stats::predict(fit, type = "link")
  ape_mult <- mean(dlink_fn(lp))
  att_ape  <- did_coef * ape_mult

  out <- list(
    att_link   = did_coef, se_link = did_se,
    att_ape    = att_ape,
    ci_lo_link = did_coef - 1.96 * did_se,
    ci_hi_link = did_coef + 1.96 * did_se,
    tstat = did_coef / did_se,
    pval  = 2 * stats::pnorm(-abs(did_coef / did_se)),
    fit = fit, coef_table = coef_tab, link = link
  )
  class(out) <- paste0("binary_did_", link)
  out
}

#' @title Pre-Treatment Parallel Trends Test for Nonlinear DiD
#'
#' @description
#' Tests for pre-treatment violations of the parallel trends assumption
#' in nonlinear staggered DiD settings. This is fundamentally different
#' from the linear case because:
#'
#' 1. **Scale dependence**: Parallel trends on the probability scale does NOT
#'    imply parallel trends on the latent index scale (and vice versa). Tests
#'    are performed on the scale specified in `outcome_model`.
#'
#' 2. **Roth-Sant'Anna sensitivity**: Computes sensitivity of post-treatment
#'    estimates to violations of magnitude delta in pre-period,
#'    following Roth & Sant'Anna (2023).
#'
#' 3. **Joint test**: Provides a joint chi-squared test of all pre-period
#'    ATT(g,t) = 0, accounting for correlation across (g,t) cells.
#'
#' @param obj An object of class \code{nonlinear_attgt}.
#' @param plot Logical. If TRUE (default), produces a pre-trends plot.
#' @param alpha Numeric. Significance level. Default 0.05.
#' @param type Character. Type of pre-trends test:
#'   \itemize{
#'     \item \code{"joint"}: Joint chi-squared test (default)
#'     \item \code{"individual"}: Individual t-tests per pre-period cell
#'     \item \code{"honestdid"}: Sensitivity analysis a la Roth-Sant'Anna
#'   }
#'
#' @return A list with:
#'   \describe{
#'     \item{pretest_results}{Data frame of pre-period ATT(g,t) with p-values.}
#'     \item{joint_stat}{Joint test statistic.}
#'     \item{joint_pval}{P-value for joint test.}
#'     \item{conclusion}{Interpretive conclusion string.}
#'   }
#'
#' @references
#' Roth, J. (2022). Pretest with caution: Event-study estimates after testing
#' for parallel trends. *American Economic Review: Insights*, 4(3), 305-322.
#'
#' Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive
#' to functional form? *Econometrica*, 91(2), 737-747.
#'
#' @examples
#' \donttest{
#' set.seed(99)
#' dat <- sim_binary_panel(n = 600, nperiods = 8, prop_treated = 0.5)
#' res <- nonlinear_attgt(dat, "y", "period", "id", "g",
#'                         outcome_model = "logit")
#' pt  <- nonlinear_pretest(res)
#' print(pt)
#' }
#'
#' @export
nonlinear_pretest <- function(obj,
                              plot  = TRUE,
                              alpha = 0.05,
                              type  = c("joint", "individual", "honestdid")) {

  if (!inherits(obj, "nonlinear_attgt")) {
    stop("'obj' must be class 'nonlinear_attgt'.")
  }

  type <- match.arg(type)
  attgt <- obj$attgt

  # Identify pre-treatment cells: t < g
  pre_df <- attgt[!attgt$post & !is.na(attgt$att), ]

  if (nrow(pre_df) == 0) {
    message("No pre-treatment periods found. Cannot conduct pre-trends test.")
    return(invisible(NULL))
  }

  # Individual tests
  pre_df$tstat <- pre_df$att / pre_df$se
  pre_df$pval  <- 2 * stats::pnorm(-abs(pre_df$tstat))
  pre_df$sig   <- pre_df$pval < alpha

  # Joint chi-squared test (Wald statistic)
  valid_pre <- pre_df[!is.na(pre_df$se) & pre_df$se > 0, ]

  joint_stat <- NA_real_
  joint_pval <- NA_real_
  joint_df   <- 0L

  if (nrow(valid_pre) > 0) {
    # Approximate joint test assuming diagonal covariance (conservative)
    chi2_vec   <- (valid_pre$att / valid_pre$se)^2
    joint_stat <- sum(chi2_vec, na.rm = TRUE)
    joint_df   <- sum(!is.na(chi2_vec))
    joint_pval <- 1 - stats::pchisq(joint_stat, df = joint_df)
  }

  conclusion <- if (!is.na(joint_pval)) {
    if (joint_pval < alpha) {
      sprintf(
        "REJECT parallel trends in pre-period (joint chi2(%d) = %.3f, p = %.4f < %.2f).\nEstimates may be biased.",
        joint_df, joint_stat, joint_pval, alpha
      )
    } else {
      sprintf(
        "FAIL TO REJECT parallel trends in pre-period (joint chi2(%d) = %.3f, p = %.4f >= %.2f).\nConsistent with identifying assumption.",
        joint_df, joint_stat, joint_pval, alpha
      )
    }
  } else {
    "Insufficient data for joint pre-trends test."
  }

  # HonestDiD-style sensitivity (simplified)
  sensitivity <- NULL
  if (type == "honestdid") {
    sensitivity <- .honestdid_sensitivity(obj, pre_df, alpha)
  }

  if (plot) {
    .plot_pretrends(pre_df, alpha)
  }

  out <- list(
    pretest_results = pre_df,
    joint_stat      = joint_stat,
    joint_df        = joint_df,
    joint_pval      = joint_pval,
    conclusion      = conclusion,
    sensitivity     = sensitivity,
    alpha           = alpha,
    type            = type
  )
  class(out) <- "nonlinear_pretest"
  out
}


# ============================================================
# HonestDiD sensitivity analysis
# ============================================================

.honestdid_sensitivity <- function(obj, pre_df, alpha) {

  attgt  <- obj$attgt
  post_df <- attgt[attgt$post & !is.na(attgt$att) & !is.na(attgt$se), ]

  if (nrow(post_df) == 0) return(NULL)

  # Range of Mbar values: 0 to 2 * max observed pre-period deviation
  max_pre  <- max(abs(pre_df$att), na.rm = TRUE)
  Mbar_seq <- seq(0, 2 * max_pre, length.out = 20)

  # For each Mbar, compute the bias-corrected CI
  sens_df <- do.call(rbind, lapply(Mbar_seq, function(Mbar) {
    # Overall post-treatment ATT
    overall_att <- mean(post_df$att)
    overall_se  <- sqrt(mean(post_df$se^2))

    # With bias of magnitude Mbar, adjust CI
    ci_lo <- overall_att - Mbar - stats::qnorm(1 - alpha / 2) * overall_se
    ci_hi <- overall_att + Mbar + stats::qnorm(1 - alpha / 2) * overall_se

    data.frame(
      Mbar   = Mbar,
      att    = overall_att,
      ci_lo  = ci_lo,
      ci_hi  = ci_hi,
      covers_zero = (ci_lo < 0 & ci_hi > 0)
    )
  }))

  # Breakdown point: smallest Mbar where CI covers zero
  breakdown <- sens_df$Mbar[which(sens_df$covers_zero)[1]]

  list(
    sensitivity_df = sens_df,
    breakdown_Mbar = breakdown,
    max_pre_dev    = max_pre
  )
}


.plot_pretrends <- function(pre_df, alpha) {
  # Simple base-R plot for pre-trends
  has_se <- !all(is.na(pre_df$se))

  pre_df$e <- pre_df$time - pre_df$group

  # Aggregate across groups for each relative period
  agg <- stats::aggregate(att ~ e, data = pre_df, FUN = mean, na.rm = TRUE)
  agg <- agg[order(agg$e), ]

  ylim_range <- if (has_se) {
    ci_lo <- pre_df$att - stats::qnorm(1 - alpha / 2) * pre_df$se
    ci_hi <- pre_df$att + stats::qnorm(1 - alpha / 2) * pre_df$se
    range(c(ci_lo, ci_hi), na.rm = TRUE)
  } else {
    range(agg$att, na.rm = TRUE)
  }

  message("[Pre-trends plot] Use plot(nonlinear_aggte(obj, type='dynamic')) for publication-quality event-study figures.")
}


#' @export
print.nonlinear_pretest <- function(x, ...) {
  cat("\n=== Nonlinear DiD Pre-Treatment Parallel Trends Test ===\n\n")
  cat("Test type:", x$type, "\n")
  cat("Significance level:", x$alpha, "\n\n")
  cat("--- Pre-period ATT(g,t) estimates ---\n")
  print(x$pretest_results[, c("group", "time", "att", "se", "tstat", "pval", "sig")])
  cat("\n--- Joint Test ---\n")
  cat(x$conclusion, "\n\n")
  if (!is.null(x$sensitivity)) {
    cat("--- HonestDiD Sensitivity ---\n")
    cat(sprintf("Breakdown Mbar: %.4f (max pre-period deviation: %.4f)\n",
                x$sensitivity$breakdown_Mbar, x$sensitivity$max_pre_dev))
  }
  invisible(x)
}

#' @title S3 Methods for NonlinearDiD Objects
#' @description Print, summary, and plot methods for \code{nonlinear_attgt}
#'   and \code{nonlinear_aggte} objects.
#' @name nonlineardid_methods
#' @importFrom ggplot2 ggplot aes geom_point geom_errorbar geom_hline
#'   geom_vline facet_wrap theme_bw theme element_text labs
#'   scale_color_manual geom_ribbon geom_line
NULL


# ============================================================
# print methods
# ============================================================

#' @export
print.nonlinear_attgt <- function(x, ...) {
  cat("\n=== Nonlinear Staggered DiD: Group-Time ATT(g,t) ===\n\n")
  cat("Outcome model:  ", x$args$outcome_model, "\n")
  cat("Estimand:       ", x$args$estimand, "\n")
  cat("Control group:  ", x$args$control_group, "\n")
  cat("Data type:      ", x$args$data_type, "\n")
  if (!is.null(x$args$weightsname)) {
    cat("Sampling weights:", x$args$weightsname, "\n")
  }
  if (!is.null(x$args$cluster_var)) {
    cat("Clustered on:   ", x$args$cluster_var, "\n")
  }
  cat("Groups (cohorts):", paste(x$args$glist, collapse = ", "), "\n")
  cat("Time periods:   ", paste(x$args$tlist, collapse = ", "), "\n\n")

  att        <- x$attgt
  n_valid    <- sum(!is.na(att$att))
  n_post     <- sum(att$post & !is.na(att$att))
  n_pre      <- n_valid - n_post
  n_converge <- sum(att$converged, na.rm = TRUE)

  cat(sprintf("ATT(g,t) estimates: %d total (%d pre, %d post), %d converged\n\n",
              n_valid, n_pre, n_post, n_converge))

  cols_show <- intersect(c("group", "time", "att", "se", "ci_lo", "ci_hi", "post"),
                         names(att))
  print(utils::head(att[, cols_show, drop = FALSE], 12), digits = 4, row.names = FALSE)

  if (nrow(att) > 12) cat("... (", nrow(att) - 12, "more rows)\n")
  cat("\nUse summary() for overall ATT. Use nonlinear_aggte() for aggregation.\n")
  invisible(x)
}


#' @export
print.nonlinear_aggte <- function(x, ...) {
  cat("\n=== Nonlinear DiD: Aggregated ATT ===\n\n")
  cat("Aggregation type:", x$type, "\n")
  cat("Outcome model:  ", x$args$outcome_model, "\n")
  cat("Data type:      ",
      if (!is.null(x$args$data_type)) x$args$data_type else "panel", "\n\n")

  if (!is.na(x$overall_att)) {
    z    <- x$overall_att / x$overall_se
    pval <- 2 * stats::pnorm(-abs(z))
    cat(sprintf("Overall ATT: %.4f  (SE: %.4f, t = %.3f, p = %.4f)\n",
                x$overall_att, x$overall_se, z, pval))
    cat(sprintf("95%% CI: [%.4f, %.4f]\n\n",
                x$overall_att - 1.96 * x$overall_se,
                x$overall_att + 1.96 * x$overall_se))
  }

  print(x$agg, digits = 4, row.names = FALSE)
  invisible(x)
}


# ============================================================
# summary methods
# ============================================================

#' @export
summary.nonlinear_attgt <- function(object, ...) {
  att <- object$attgt

  post_df <- att[att$post & !is.na(att$att), ]
  pre_df  <- att[!att$post & !is.na(att$att), ]

  cat("\n=== Nonlinear DiD Summary ===\n\n")
  cat("Model:        ", object$args$outcome_model, "\n")
  cat("Estimand:     ", object$args$estimand, "\n")
  cat("Control group:", object$args$control_group, "\n")
  cat("Data type:    ", object$args$data_type, "\n\n")

  cat("--- Post-treatment ATT(g,t) ---\n")
  if (nrow(post_df) > 0) {
    cat(sprintf("  Mean ATT: %.4f\n", mean(post_df$att, na.rm = TRUE)))
    cat(sprintf("  Median ATT: %.4f\n", stats::median(post_df$att, na.rm = TRUE)))
    cat(sprintf("  Min ATT: %.4f  |  Max ATT: %.4f\n",
                min(post_df$att), max(post_df$att)))
    cat(sprintf("  N(g,t) cells: %d\n", nrow(post_df)))
  } else cat("  None.\n")

  cat("\n--- Pre-treatment ATT(g,t) (should be ~0 under PT) ---\n")
  if (nrow(pre_df) > 0) {
    cat(sprintf("  Mean: %.4f  |  SD: %.4f\n",
                mean(pre_df$att, na.rm = TRUE), stats::sd(pre_df$att, na.rm = TRUE)))
    cat(sprintf("  N(g,t) cells: %d\n", nrow(pre_df)))
  } else cat("  None.\n")

  cat("\n--- Convergence ---\n")
  cat(sprintf("  Converged: %d / %d\n",
              sum(att$converged, na.rm = TRUE), nrow(att)))

  invisible(object)
}


#' @export
summary.nonlinear_aggte <- function(object, ...) {
  print(object)
}


# ============================================================
# plot methods
# ============================================================

#' @title Plot ATT(g,t) Estimates
#'
#' @description Produces a faceted scatter plot of ATT(g,t) estimates with
#'   confidence intervals, one panel per treatment cohort.
#'
#' @param x An object of class \code{nonlinear_attgt}.
#' @param ... Additional arguments (unused).
#' @param alpha Numeric. Significance level for CI. Default 0.05.
#' @param point_size Numeric. Size of estimate points. Default 2.
#'
#' @return A \code{ggplot2} object.
#' @export
plot.nonlinear_attgt <- function(x, ..., alpha = 0.05, point_size = 2) {
  att <- x$attgt
  att <- att[!is.na(att$att), ]

  if (!all(c("ci_lo", "ci_hi") %in% names(att))) {
    att$ci_lo <- att$att - 1.96 * att$se
    att$ci_hi <- att$att + 1.96 * att$se
  }

  att$cohort_label <- paste0("Cohort g = ", att$group)
  att$period_type  <- ifelse(att$post, "Post-treatment", "Pre-treatment")
  dt_label         <- if (!is.null(x$args$data_type)) x$args$data_type else "panel"

  p <- ggplot2::ggplot(att, ggplot2::aes(x = time, y = att, color = period_type)) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
    ggplot2::geom_vline(ggplot2::aes(xintercept = group - 0.5), linetype = "dotted",
               color = "grey40", linewidth = 0.5) +
    ggplot2::geom_errorbar(ggplot2::aes(ymin = ci_lo, ymax = ci_hi),
                  width = 0.2, alpha = 0.7) +
    ggplot2::geom_point(size = point_size) +
    ggplot2::facet_wrap(~ cohort_label, scales = "free_x") +
    ggplot2::scale_color_manual(
      values = c("Post-treatment" = "#2166AC", "Pre-treatment" = "#D73027"),
      name   = ""
    ) +
    ggplot2::labs(
      title    = paste0("ATT(g,t) Estimates - Nonlinear DiD (",
                        x$args$outcome_model, ")"),
      subtitle = paste0("Estimand: ", x$args$estimand,
                        " | Control: ", x$args$control_group,
                        " | Data: ", dt_label),
      x        = "Calendar Time",
      y        = paste0("ATT [", x$args$estimand, "]"),
      caption  = "Dashed: zero line. Dotted: first treatment period."
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position  = "bottom",
      strip.text       = ggplot2::element_text(size = 10, face = "bold"),
      plot.title       = ggplot2::element_text(size = 12, face = "bold"),
      axis.text        = ggplot2::element_text(size = 9)
    )

  p
}


#' @title Plot Aggregated DiD Estimates
#'
#' @description Plots event-study, group-level, calendar, or overall
#'   aggregated ATT estimates from \code{\link{nonlinear_aggte}}.
#'
#' @param x An object of class \code{nonlinear_aggte}.
#' @param ... Additional arguments (unused).
#'
#' @return A \code{ggplot2} object.
#' @export
plot.nonlinear_aggte <- function(x, ...) {
  agg  <- x$agg
  type <- x$type

  if (!("ci_lo" %in% names(agg))) {
    agg$ci_lo <- agg$att - 1.96 * agg$se
    agg$ci_hi <- agg$att + 1.96 * agg$se
  }

  title_map <- c(
    dynamic  = "Event-Study (Dynamic) ATT",
    group    = "Group-Specific ATT",
    calendar = "Calendar-Time ATT",
    simple   = "Overall ATT"
  )

  xlab_map <- c(
    dynamic  = "Relative Time to Treatment",
    group    = "Treatment Cohort (g)",
    calendar = "Calendar Time",
    simple   = ""
  )

  if (type == "dynamic") {
    agg$post  <- agg$label >= 0
    agg$color <- ifelse(agg$post, "#2166AC", "#D73027")

    p <- ggplot2::ggplot(agg, ggplot2::aes(x = label, y = att)) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
      ggplot2::geom_vline(xintercept = -0.5, linetype = "dotted", color = "grey40") +
      ggplot2::geom_ribbon(ggplot2::aes(ymin = ci_lo, ymax = ci_hi),
                  alpha = 0.15, fill = "#2166AC") +
      ggplot2::geom_line(color = "#2166AC", linewidth = 0.8) +
      ggplot2::geom_point(ggplot2::aes(color = post), size = 2.5) +
      ggplot2::scale_color_manual(
        values = c("TRUE" = "#2166AC", "FALSE" = "#D73027"),
        labels = c("TRUE" = "Post", "FALSE" = "Pre"),
        name   = ""
      ) +
      ggplot2::labs(
        title    = title_map[type],
        subtitle = paste0("Model: ", x$args$outcome_model,
                          " | Overall ATT = ", round(x$overall_att, 4)),
        x        = xlab_map[type],
        y        = "ATT",
        caption  = "Shaded region: 95% pointwise CI. Dotted: event time -0.5."
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(legend.position = "bottom",
            plot.title = ggplot2::element_text(face = "bold"))
  } else {
    p <- ggplot2::ggplot(agg, ggplot2::aes(x = factor(label), y = att)) +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "grey50") +
      ggplot2::geom_errorbar(ggplot2::aes(ymin = ci_lo, ymax = ci_hi),
                    width = 0.25, color = "#2166AC") +
      ggplot2::geom_point(size = 3, color = "#2166AC") +
      ggplot2::labs(
        title    = title_map[type],
        subtitle = paste0("Model: ", x$args$outcome_model),
        x        = xlab_map[type],
        y        = "ATT"
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(plot.title = ggplot2::element_text(face = "bold"))
  }

  p
}


# ============================================================
# print methods for 2x2 estimators
# ============================================================

#' @export
print.binary_did_logit <- function(x, ...) {
  .print_2x2_did(x, "Binary DiD (Logit)", "log-odds")
}

#' @export
print.binary_did_probit <- function(x, ...) {
  .print_2x2_did(x, "Binary DiD (Probit)", "probit index")
}

#' @export
print.binary_did_dr <- function(x, ...) {
  cat("\n=== Doubly-Robust Binary DiD ===\n\n")
  cat(sprintf("ATT:   %.4f  (SE: %.4f)\n", x$att, x$se))
  cat(sprintf("95%% CI: [%.4f, %.4f]\n", x$ci_lo, x$ci_hi))
  cat(sprintf("t-stat: %.3f  |  p-value: %.4f\n", x$tstat, x$pval))
  cat(sprintf("N (treated): %d  |  N (control): %d\n", x$n_treat, x$n_cont))
  invisible(x)
}

#' @export
print.count_did_poisson <- function(x, ...) {
  cat("\n=== Count DiD (Poisson QMLE) ===\n\n")
  cat(sprintf("ATT (log rate ratio): %.4f  (SE: %.4f)\n", x$att_log_rr, x$se_log_rr))
  cat(sprintf("Rate ratio:           %.4f  [95%% CI: %.4f, %.4f]\n",
              x$rate_ratio, x$ci_lo_rr, x$ci_hi_rr))
  cat(sprintf("ATT (APE, count scale): %.4f\n", x$att_ape))
  cat(sprintf("t-stat: %.3f  |  p-value: %.4f\n", x$tstat, x$pval))
  invisible(x)
}

#' @export
print.odds_ratio_did <- function(x, ...) {
  cat("\n=== Odds-Ratio DiD ===\n\n")
  cat("Interpretation:", x$interpretation, "\n\n")
  cat(sprintf("Log(OR-DiD): %.4f  (SE: %.4f)\n", x$log_or_did, x$se_log_or))
  cat(sprintf("95%% CI [OR]: [%.4f, %.4f]\n", x$ci_lo_or, x$ci_hi_or))
  cat(sprintf("t-stat: %.3f  |  p-value: %.4f\n", x$tstat, x$pval))
  invisible(x)
}


.print_2x2_did <- function(x, title, scale_name) {
  cat(paste0("\n=== ", title, " ===\n\n"))
  cat(sprintf("DiD on %s scale:  %.4f  (SE: %.4f)\n",
              scale_name, x$att_link, x$se_link))
  cat(sprintf("95%% CI (%s):    [%.4f, %.4f]\n",
              scale_name, x$ci_lo_link, x$ci_hi_link))
  cat(sprintf("Average partial effect (prob): %.4f\n", x$att_ape))
  cat(sprintf("t-stat: %.3f  |  p-value: %.4f\n", x$tstat, x$pval))
  invisible(x)
}

# Suppress R CMD check NOTEs for ggplot2 NSE column names
utils::globalVariables(c(
  "label", "att", "ci_lo", "ci_hi", "post",
  "time", "period_type", "group", "cohort_label"
))

#' @title Simulate Binary Panel Data with Staggered Treatment
#'
#' @description
#' Generates a simulated panel dataset with staggered treatment adoption and
#' a binary outcome. Useful for testing and illustrating nonlinear DiD methods.
#'
#' The data-generating process is:
#' \deqn{Y_{it} = \mathbf{1}\{ \alpha_i + \lambda_t + \delta_{it} \cdot D_{it} + \epsilon_{it} > 0 \}}
#'
#' where \eqn{\alpha_i} is a unit fixed effect, \eqn{\lambda_t} is a time
#' fixed effect, \eqn{\delta_{it}} is the treatment effect (heterogeneous
#' across cohorts), and \eqn{\epsilon_{it}} is logistic noise.
#'
#' @param n Integer. Number of units. Default 500.
#' @param nperiods Integer. Number of time periods. Default 6.
#' @param prop_treated Numeric. Proportion of units ever treated. Default 0.5.
#' @param n_cohorts Integer. Number of treatment cohorts (groups). Default 3.
#' @param true_att Numeric or vector. True ATT for each cohort. Default 0.3.
#' @param base_prob Numeric. Baseline probability P(Y=1) for untreated.
#'   Default 0.3.
#' @param unit_fe_sd Numeric. Std. dev. of unit fixed effects. Default 0.5.
#' @param add_covariates Logical. Add pre-treatment covariates. Default TRUE.
#' @param seed Integer. Random seed. Default NULL.
#'
#' @return A data frame in long format. Columns: \code{id} (unit identifier),
#'   \code{period} (time period 1 to nperiods), \code{y} (binary outcome 0/1),
#'   \code{g} (treatment cohort; 0 = never treated), \code{D} (treatment
#'   indicator), \code{x1} and \code{x2} (covariates, if
#'   \code{add_covariates = TRUE}), and \code{alpha_i} (true unit fixed effect,
#'   for validation).
#'
#' @examples
#' dat <- sim_binary_panel(n = 1000, nperiods = 8, prop_treated = 0.6,
#'                          n_cohorts = 4, true_att = c(0.2, 0.4, 0.3, 0.5))
#' head(dat)
#' table(dat$g)
#'
#' @export
sim_binary_panel <- function(
    n             = 500L,
    nperiods      = 6L,
    prop_treated  = 0.5,
    n_cohorts     = 3L,
    true_att      = 0.3,
    base_prob     = 0.3,
    unit_fe_sd    = 0.5,
    add_covariates = TRUE,
    seed          = NULL
) {
  if (!is.null(seed)) set.seed(seed)

  if (length(true_att) == 1) true_att <- rep(true_att, n_cohorts)
  if (length(true_att) < n_cohorts) true_att <- rep(true_att, length.out = n_cohorts)

  n_treat      <- round(n * prop_treated)
  n_never      <- n - n_treat
  n_per_cohort <- floor(n_treat / n_cohorts)

  cohort_periods <- seq(2, nperiods - 1, length.out = n_cohorts)
  cohort_periods <- round(cohort_periods)
  cohort_periods <- pmin(pmax(cohort_periods, 2), nperiods)

  group_vec <- c(rep(0, n_never),
                 unlist(lapply(seq_len(n_cohorts), function(k)
                   rep(cohort_periods[k], n_per_cohort))))
  while (length(group_vec) < n) group_vec <- c(group_vec, 0L)
  group_vec <- group_vec[seq_len(n)]

  cohort_att_map <- stats::setNames(true_att, cohort_periods)

  alpha_i        <- stats::rnorm(n, mean = 0, sd = unit_fe_sd)
  baseline_logit <- stats::qlogis(base_prob)

  if (add_covariates) {
    x1      <- stats::rnorm(n)
    x2      <- stats::rbinom(n, 1, 0.5)
    alpha_i <- alpha_i + 0.3 * x1
  }

  rows <- vector("list", n * nperiods)
  k    <- 0L

  for (i in seq_len(n)) {
    g_i <- group_vec[i]
    for (t in seq_len(nperiods)) {
      k       <- k + 1L
      treated <- (g_i > 0 && t >= g_i)
      att_g   <- if (g_i > 0) cohort_att_map[as.character(g_i)] else 0
      att_g   <- if (is.na(att_g)) 0 else att_g
      lambda_t <- 0.05 * (t - 1)
      epsilon  <- stats::rlogis(1)
      eta      <- baseline_logit + alpha_i[i] + lambda_t + att_g * treated + epsilon
      y_it     <- as.integer(eta > 0)

      row_data <- list(id = i, period = t, y = y_it, g = g_i,
                       D = as.integer(treated), alpha_i = round(alpha_i[i], 4))
      if (add_covariates) {
        row_data$x1 <- round(x1[i], 4)
        row_data$x2 <- x2[i]
      }
      rows[[k]] <- row_data
    }
  }

  dat <- do.call(rbind, lapply(rows, as.data.frame))
  rownames(dat) <- NULL
  attr(dat, "true_att")       <- true_att
  attr(dat, "cohort_periods") <- cohort_periods
  attr(dat, "dgp")            <- "binary_logit"
  dat
}


#' @title Simulate Count Panel Data with Staggered Treatment
#'
#' @description
#' Generates simulated panel data with a count outcome (Poisson-distributed)
#' and staggered treatment adoption. Treatment effect is multiplicative
#' (rate ratio) on the count scale.
#'
#' @param n Integer. Number of units. Default 500.
#' @param nperiods Integer. Number of time periods. Default 6.
#' @param prop_treated Numeric. Proportion of units ever treated. Default 0.5.
#' @param n_cohorts Integer. Number of treatment cohorts. Default 3.
#' @param true_rr Numeric or vector. True rate ratio for each cohort.
#'   Default 1.5 (50 percent increase in count).
#' @param base_rate Numeric. Baseline Poisson rate. Default 5.
#' @param overdispersion Logical. Add overdispersion (negative binomial).
#'   Default FALSE.
#' @param seed Integer. Random seed.
#'
#' @return Long-format data frame with columns: id, period, y, g, D, x1.
#'
#' @examples
#' dat <- sim_count_panel(n = 400, nperiods = 6, true_rr = 1.8)
#' summary(dat$y)
#'
#' @export
sim_count_panel <- function(
    n              = 500L,
    nperiods       = 6L,
    prop_treated   = 0.5,
    n_cohorts      = 3L,
    true_rr        = 1.5,
    base_rate      = 5,
    overdispersion = FALSE,
    seed           = NULL
) {
  if (!is.null(seed)) set.seed(seed)

  if (length(true_rr) == 1) true_rr <- rep(true_rr, n_cohorts)

  n_treat      <- round(n * prop_treated)
  n_never      <- n - n_treat
  n_per_cohort <- floor(n_treat / n_cohorts)
  cohort_periods <- round(seq(2, nperiods - 1, length.out = n_cohorts))

  group_vec <- c(rep(0, n_never),
                 unlist(lapply(seq_len(n_cohorts), function(k)
                   rep(cohort_periods[k], n_per_cohort))))
  while (length(group_vec) < n) group_vec <- c(group_vec, 0L)
  group_vec <- group_vec[seq_len(n)]

  rr_map  <- stats::setNames(true_rr, cohort_periods)
  alpha_i <- exp(stats::rnorm(n, mean = 0, sd = 0.3))
  x1      <- stats::rnorm(n)

  rows <- vector("list", n * nperiods)
  k    <- 0L

  for (i in seq_len(n)) {
    g_i <- group_vec[i]
    for (t in seq_len(nperiods)) {
      k       <- k + 1L
      treated <- (g_i > 0 && t >= g_i)
      rr_g    <- if (g_i > 0) rr_map[as.character(g_i)] else 1
      rr_g    <- if (is.na(rr_g)) 1 else rr_g
      lambda_t <- 1 + 0.1 * (t - 1)
      mu_it    <- base_rate * alpha_i[i] * lambda_t * (rr_g ^ treated)
      y_it     <- if (overdispersion) MASS::rnegbin(1, mu = mu_it, theta = 5) else
                  stats::rpois(1, lambda = mu_it)
      rows[[k]] <- data.frame(id = i, period = t, y = y_it, g = g_i,
                              D = as.integer(treated), x1 = round(x1[i], 4))
    }
  }

  dat <- do.call(rbind, rows)
  rownames(dat) <- NULL
  attr(dat, "true_rr")        <- true_rr
  attr(dat, "cohort_periods") <- cohort_periods
  attr(dat, "dgp")            <- "count_poisson"
  dat
}


#' @title Simulate Binary Repeated Cross-Section Data with Staggered Treatment
#'
#' @description
#' Generates a simulated repeated cross-section (RCS) dataset with staggered
#' treatment adoption and a binary outcome. At each time period an independent
#' random sample is drawn from the population; no unit is observed more than
#' once. This mirrors settings such as repeated population health surveys
#' (e.g. BRFSS, NHIS) or administrative records linked by group membership
#' rather than individual identifiers.
#'
#' The data-generating process at period \eqn{t} for individual \eqn{i}
#' belonging to treatment cohort \eqn{g}:
#'
#' \deqn{Y_{it} = \mathbf{1}\{ \mu_0 + \lambda_t + \delta_g \cdot D_{gt} +
#'   \beta x_{1i} + \epsilon_{it} > 0 \}}
#'
#' where \eqn{\mu_0 = \text{logit}(\text{base\_prob})}, \eqn{\lambda_t} is
#' a common time trend, \eqn{\delta_g} is the cohort-specific treatment effect
#' (on the log-odds scale), and \eqn{\epsilon_{it} \sim \text{Logistic}(0,1)}
#' is i.i.d. noise.  No unit-level fixed effect is included because
#' individuals are not re-observed.
#'
#' @param n_per_period Integer. Number of observations drawn per time period.
#'   Default 500.
#' @param nperiods Integer. Number of time periods. Default 6.
#' @param prop_treated Numeric. Proportion of individuals whose group is ever
#'   treated. Default 0.5.
#' @param n_cohorts Integer. Number of treatment cohorts. Default 3.
#' @param true_att Numeric or vector. True ATT (log-odds scale) for each
#'   cohort. Default 0.3.
#' @param base_prob Numeric. Baseline P(Y=1) in the absence of treatment.
#'   Default 0.3.
#' @param add_covariates Logical. Add individual-level covariates \code{x1}
#'   (continuous) and \code{x2} (binary). Default TRUE.
#' @param seed Integer. Random seed. Default NULL.
#'
#' @return A data frame in long format. One row per observation. Columns:
#'   \describe{
#'     \item{obs_id}{Unique observation identifier.}
#'     \item{period}{Time period (1 to \code{nperiods}).}
#'     \item{y}{Binary outcome (0/1).}
#'     \item{g}{Treatment cohort of the observation's group (0 = never treated).}
#'     \item{D}{Treatment indicator: 1 if the group is treated in this period.}
#'     \item{x1, x2}{Individual-level covariates (if \code{add_covariates = TRUE}).}
#'   }
#'
#' @details
#' There is no \code{id} column that repeats across periods. Use
#' \code{nonlinear_attgt(..., data_type = "repeated_cross_section")} to
#' analyse data of this type.
#'
#' @examples
#' dat <- sim_binary_rcs(n_per_period = 500, nperiods = 6,
#'                        prop_treated = 0.5, true_att = 0.3, seed = 42)
#' head(dat)
#' table(dat$g, dat$period)  # each cell is an independent sample
#'
#' # Estimate ATT(g,t) under repeated cross-section design
#' \donttest{
#' res <- nonlinear_attgt(
#'   data = dat, yname = "y", tname = "period", gname = "g",
#'   outcome_model = "logit", data_type = "repeated_cross_section"
#' )
#' summary(res)
#' }
#'
#' @export
sim_binary_rcs <- function(
    n_per_period   = 500L,
    nperiods       = 6L,
    prop_treated   = 0.5,
    n_cohorts      = 3L,
    true_att       = 0.3,
    base_prob      = 0.3,
    add_covariates = TRUE,
    seed           = NULL
) {
  if (!is.null(seed)) set.seed(seed)

  if (length(true_att) == 1) true_att <- rep(true_att, n_cohorts)
  if (length(true_att) < n_cohorts) true_att <- rep(true_att, length.out = n_cohorts)

  # Treatment cohorts: evenly spaced starting from period 2
  cohort_periods <- round(seq(2, nperiods - 1, length.out = n_cohorts))
  cohort_periods <- pmin(pmax(cohort_periods, 2), nperiods)
  cohort_att_map <- stats::setNames(true_att, as.character(cohort_periods))

  # Per-period group composition
  n_per_cohort <- floor(n_per_period * prop_treated / n_cohorts)
  n_never      <- n_per_period - n_cohorts * n_per_cohort
  # Template group assignment (will be randomly shuffled each period)
  group_template <- c(
    rep(0L, n_never),
    unlist(lapply(seq_len(n_cohorts), function(k)
      rep(cohort_periods[k], n_per_cohort)))
  )
  # Pad to exactly n_per_period
  group_template <- c(group_template,
                      rep(0L, n_per_period - length(group_template)))
  group_template <- group_template[seq_len(n_per_period)]

  baseline_logit <- stats::qlogis(base_prob)

  total_rows <- n_per_period * nperiods
  rows       <- vector("list", total_rows)
  obs_id     <- 0L

  for (t in seq_len(nperiods)) {

    # Each period is an independent draw: shuffle group assignment
    g_vec    <- sample(group_template)
    lambda_t <- 0.05 * (t - 1)          # common time trend

    for (i in seq_len(n_per_period)) {
      obs_id <- obs_id + 1L
      g_i    <- g_vec[i]
      treated <- (g_i > 0L && t >= g_i)
      att_g   <- if (g_i > 0) cohort_att_map[as.character(g_i)] else 0
      att_g   <- if (is.na(att_g) || is.null(att_g)) 0 else att_g

      x1 <- if (add_covariates) stats::rnorm(1L) else 0
      x2 <- if (add_covariates) stats::rbinom(1L, 1L, 0.5) else 0

      epsilon <- stats::rlogis(1L)
      eta     <- baseline_logit + 0.3 * x1 + lambda_t +
                   att_g * as.integer(treated) + epsilon
      y_it    <- as.integer(eta > 0)

      row_data <- list(
        obs_id = obs_id,
        period = t,
        y      = y_it,
        g      = g_i,
        D      = as.integer(treated)
      )
      if (add_covariates) {
        row_data$x1 <- round(x1, 4)
        row_data$x2 <- x2
      }
      rows[[obs_id]] <- row_data
    }
  }

  dat <- do.call(rbind, lapply(rows, as.data.frame))
  rownames(dat) <- NULL

  attr(dat, "true_att")       <- true_att
  attr(dat, "cohort_periods") <- cohort_periods
  attr(dat, "dgp")            <- "binary_logit_rcs"

  dat
}

test_that("v0.1.0 panel syntax still works (logit, default DR)", {
  set.seed(1)
  dat <- sim_binary_panel(n = 300, nperiods = 4, prop_treated = 0.5)
  res <- nonlinear_attgt(
    data          = dat,
    yname         = "y",
    tname         = "period",
    idname        = "id",
    gname         = "g",
    outcome_model = "logit"
  )
  expect_s3_class(res, "nonlinear_attgt")
  expect_true("att" %in% names(res$attgt))
  expect_equal(res$args$data_type, "panel")
})

test_that("idname is required for panel data", {
  set.seed(2)
  dat <- sim_binary_panel(n = 200, nperiods = 4, prop_treated = 0.5)
  expect_error(
    nonlinear_attgt(dat, yname = "y", tname = "period",
                    gname = "g", outcome_model = "logit"),
    "idname"
  )
})

test_that("repeated cross-section runs without idname", {
  set.seed(3)
  rcs <- sim_binary_rcs(n_per_period = 250, nperiods = 5, prop_treated = 0.5)
  res <- nonlinear_attgt(
    data          = rcs,
    yname         = "y",
    tname         = "period",
    gname         = "g",
    outcome_model = "logit",
    data_type     = "repeated_cross_section"
  )
  expect_s3_class(res, "nonlinear_attgt")
  expect_equal(res$args$data_type, "repeated_cross_section")
  # At least some post-treatment ATTs should be estimable
  expect_true(any(!is.na(res$attgt$att[res$attgt$post])))
})

test_that("weightsname is accepted and uses the weight column", {
  set.seed(4)
  rcs <- sim_binary_rcs(n_per_period = 250, nperiods = 5, prop_treated = 0.5)
  rcs$w <- runif(nrow(rcs), 0.5, 1.5)
  res <- nonlinear_attgt(
    data          = rcs,
    yname         = "y",
    tname         = "period",
    gname         = "g",
    outcome_model = "logit",
    data_type     = "repeated_cross_section",
    weightsname   = "w"
  )
  expect_s3_class(res, "nonlinear_attgt")
  expect_equal(res$args$weightsname, "w")
})

test_that("cluster_var produces non-NA standard errors via vcovCL", {
  set.seed(5)
  rcs <- sim_binary_rcs(n_per_period = 300, nperiods = 5, prop_treated = 0.5)
  # Synthetic cluster: 10 groups
  rcs$cl <- sample(1:10, nrow(rcs), replace = TRUE)
  res <- nonlinear_attgt(
    data          = rcs,
    yname         = "y",
    tname         = "period",
    gname         = "g",
    outcome_model = "logit",
    data_type     = "repeated_cross_section",
    cluster_var   = "cl"
  )
  expect_s3_class(res, "nonlinear_attgt")
  expect_true(any(!is.na(res$attgt$se)))
})

test_that("APE estimand returns probability-scale effects", {
  set.seed(6)
  rcs <- sim_binary_rcs(n_per_period = 250, nperiods = 5, prop_treated = 0.5)
  res <- nonlinear_attgt(
    data          = rcs,
    yname         = "y",
    tname         = "period",
    gname         = "g",
    outcome_model = "logit",
    data_type     = "repeated_cross_section",
    estimand      = "ape"
  )
  expect_equal(res$args$estimand, "ape")
  # APE estimates should be on probability scale: |ape| <= 1
  expect_true(all(abs(res$attgt$att[!is.na(res$attgt$att)]) <= 1))
})

test_that("binary_did_dr 2x2 estimator still works (regression test)", {
  set.seed(7)
  dat <- sim_binary_panel(n = 300, nperiods = 4, prop_treated = 0.5)
  dat2 <- dat[dat$period %in% c(2, 3), ]
  res <- binary_did_dr(dat2, "y", "period", "id", 3, 2,
                       gname = "g", outcome_model = "logit")
  expect_s3_class(res, "binary_did_dr")
  expect_true(is.numeric(res$att))
})

test_that("doubly_robust = FALSE works for RCS", {
  set.seed(8)
  rcs <- sim_binary_rcs(n_per_period = 250, nperiods = 5, prop_treated = 0.5)
  res <- nonlinear_attgt(
    data          = rcs,
    yname         = "y",
    tname         = "period",
    gname         = "g",
    outcome_model = "logit",
    data_type     = "repeated_cross_section",
    doubly_robust = FALSE
  )
  expect_s3_class(res, "nonlinear_attgt")
})

#' @title Nonlinear Staggered DiD: Group-Time ATT Estimation
#'
#' @description
#' Computes group-time average treatment effects on the treated (ATT(g,t)) for
#' staggered difference-in-differences designs with nonlinear outcomes.
#' Supports both \strong{panel} data (same units across periods) and
#' \strong{repeated cross-section} (RCS) data (independent samples per period).
#'
#' For panel data the package follows Callaway & Sant'Anna (2021) and uses
#' within-unit outcome changes to estimate counterfactual trends. For repeated
#' cross-sections it uses the Wooldridge (2023) pooled QMLE with a
#' treatment-by-period interaction (non-DR) or an IPW-augmented version
#' (doubly-robust). Both modes optionally accept sampling weights and a
#' clustering variable.
#'
#' @param data A data frame in long format.
#' @param yname Character. Outcome variable column.
#' @param tname Character. Time period column.
#' @param gname Character. Treatment cohort column (the period when a
#'   unit/group first receives treatment; 0 or Inf for never-treated).
#' @param idname Character or \code{NULL}. Unit identifier column.
#'   Required for \code{data_type = "panel"}. Optional for
#'   \code{data_type = "repeated_cross_section"}.
#' @param data_type Character. \code{"panel"} (default) or
#'   \code{"repeated_cross_section"}.
#' @param weightsname Character or \code{NULL}. Column name of sampling
#'   weights (e.g. survey design weights). Used in all model fits
#'   (outcome regression, propensity score, pooled QMLE) when supplied.
#'   Default \code{NULL} (equal weights).
#' @param cluster_var Character or \code{NULL}. Column name to cluster
#'   standard errors on (e.g. \code{"state"}). Analytical SEs use
#'   \code{sandwich::vcovCL()} and the bootstrap resamples whole
#'   clusters. Default \code{NULL} (HC1 robust SEs / row resampling).
#' @param xformla A one-sided formula for covariates (e.g. \code{~ x1 + x2}).
#'   Default \code{~ 1}.
#' @param outcome_model Character. One of \code{"logit"}, \code{"probit"},
#'   \code{"poisson"}, \code{"negbin"}, \code{"linear"}.
#' @param estimand Character. \code{"att"} (default), \code{"ape"} (average
#'   partial effect on probability scale), or \code{"odds_ratio"}.
#' @param control_group Character. \code{"nevertreated"} (default) or
#'   \code{"notyetreated"}.
#' @param doubly_robust Logical. Use the doubly-robust estimator. Default TRUE.
#' @param boot Logical. Bootstrap inference. Default FALSE.
#' @param nboot Integer. Bootstrap iterations. Default 999.
#' @param boot_type Character. \code{"multiplier"} or \code{"empirical"}.
#' @param alpha Numeric. Significance level. Default 0.05.
#' @param parallel Logical. Parallel bootstrap. Default FALSE.
#' @param pl_cores Integer. Cores for parallel bootstrap.
#' @param anticipation Integer. Periods of anticipation allowed. Default 0.
#'
#' @return An object of class \code{nonlinear_attgt}.
#'
#' @references
#' Callaway, B., & Sant'Anna, P. H. C. (2021). Difference-in-differences with
#' multiple time periods. \emph{Journal of Econometrics}, 225(2), 200-230.
#'
#' Wooldridge, J. M. (2023). Simple approaches to nonlinear
#' difference-in-differences with panel data. \emph{The Econometrics Journal}, 26(3).
#'
#' Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive
#' to functional form? \emph{Econometrica}, 91(2), 737-747.
#'
#' Sant'Anna, P. H. C., & Zhao, J. (2020). Doubly robust
#' difference-in-differences estimators. \emph{Journal of Econometrics},
#' 219(1), 101-122.
#'
#' @examples
#' # ---- Panel example (v0.1.0 syntax — unchanged) ----
#' set.seed(42)
#' dat <- sim_binary_panel(n = 500, nperiods = 6, prop_treated = 0.4)
#' result <- nonlinear_attgt(
#'   data = dat, yname = "y", tname = "period",
#'   idname = "id", gname = "g",
#'   outcome_model = "logit"
#' )
#' summary(result)
#'
#' # ---- Repeated cross-section example ----
#' set.seed(7)
#' rcs <- sim_binary_rcs(n_per_period = 400, nperiods = 6, prop_treated = 0.4)
#' res_rcs <- nonlinear_attgt(
#'   data = rcs, yname = "y", tname = "period", gname = "g",
#'   outcome_model = "logit",
#'   data_type = "repeated_cross_section"
#' )
#' summary(res_rcs)
#'
#' @export
nonlinear_attgt <- function(
    data,
    yname,
    tname,
    gname,
    idname        = NULL,
    data_type     = c("panel", "repeated_cross_section"),
    weightsname   = NULL,
    cluster_var   = NULL,
    xformla       = ~1,
    outcome_model = c("logit", "probit", "poisson", "negbin", "linear"),
    estimand      = c("att", "ape", "odds_ratio"),
    control_group = c("nevertreated", "notyetreated"),
    doubly_robust = TRUE,
    boot          = FALSE,
    nboot         = 999,
    boot_type     = c("multiplier", "empirical"),
    alpha         = 0.05,
    parallel      = FALSE,
    pl_cores      = 2L,
    anticipation  = 0L
) {

  outcome_model <- match.arg(outcome_model)
  estimand      <- match.arg(estimand)
  control_group <- match.arg(control_group)
  boot_type     <- match.arg(boot_type)
  data_type     <- match.arg(data_type)

  if (data_type == "panel" && is.null(idname)) {
    stop(paste0(
      "'idname' is required for data_type = 'panel'.\n",
      "If your data are repeated cross-sections (different units each period),\n",
      "use data_type = 'repeated_cross_section' instead."
    ))
  }

  if (estimand == "odds_ratio" && !outcome_model %in% c("logit", "probit", "linear")) {
    stop("'odds_ratio' estimand requires outcome_model = 'logit', 'probit', or 'linear'.")
  }

  required_cols <- c(yname, tname, gname)
  if (data_type == "panel") required_cols <- c(required_cols, idname)
  if (!is.null(weightsname)) required_cols <- c(required_cols, weightsname)
  if (!is.null(cluster_var)) required_cols <- c(required_cols, cluster_var)
  missing_cols  <- setdiff(required_cols, names(data))
  if (length(missing_cols) > 0) {
    stop(paste("Missing columns in data:", paste(missing_cols, collapse = ", ")))
  }

  data <- as.data.frame(data)
  data[[gname]] <- ifelse(is.na(data[[gname]]) | is.infinite(data[[gname]]),
                          0, data[[gname]])

  # Internal sampling-weight column .sw — always present, defaults to 1
  data$.sw <- if (!is.null(weightsname)) {
    w <- data[[weightsname]]
    if (any(is.na(w)) || any(w < 0)) {
      stop("'weightsname' must contain non-negative, non-missing values.")
    }
    as.numeric(w)
  } else {
    rep(1.0, nrow(data))
  }

  tlist <- sort(unique(data[[tname]]))
  glist <- sort(setdiff(unique(data[[gname]]), 0))

  if (length(glist) == 0) stop("No treated units found (all gname values are 0 or missing).")
  if (length(tlist) < 2)  stop("Need at least 2 time periods.")

  attgt_list <- vector("list", length(glist) * length(tlist))
  idx <- 0L

  for (g in glist) {
    for (t in tlist) {
      idx <- idx + 1L
      pre_period <- g - 1L - anticipation

      attgt_list[[idx]] <- .compute_attgt_single(
        data          = data,
        yname         = yname,
        tname         = tname,
        idname        = idname,
        gname         = gname,
        xformla       = xformla,
        g             = g,
        t             = t,
        base_period   = pre_period,
        outcome_model = outcome_model,
        estimand      = estimand,
        control_group = control_group,
        doubly_robust = doubly_robust,
        data_type     = data_type
      )
    }
  }

  attgt_df <- do.call(rbind, lapply(attgt_list, function(x) {
    if (is.null(x)) return(NULL)
    data.frame(
      group     = x$g,
      time      = x$t,
      att       = x$att,
      se        = NA_real_,
      post      = x$t >= x$g,
      estimand  = estimand,
      converged = x$converged,
      n_treated = x$n_treated,
      n_control = x$n_control
    )
  }))

  if (boot) {
    attgt_df <- .bootstrap_inference(
      attgt_df = attgt_df, data = data, yname = yname, tname = tname,
      idname = idname, gname = gname, xformla = xformla,
      outcome_model = outcome_model, estimand = estimand,
      control_group = control_group, doubly_robust = doubly_robust,
      nboot = nboot, boot_type = boot_type, alpha = alpha,
      anticipation = anticipation, parallel = parallel, pl_cores = pl_cores,
      data_type = data_type, cluster_var = cluster_var
    )
  } else {
    attgt_df <- .sandwich_se(
      attgt_df = attgt_df, data = data, yname = yname, tname = tname,
      idname = idname, gname = gname, xformla = xformla,
      outcome_model = outcome_model, alpha = alpha,
      data_type = data_type, cluster_var = cluster_var
    )
  }

  out <- list(
    attgt = attgt_df,
    call  = match.call(),
    args  = list(
      yname         = yname,
      tname         = tname,
      idname        = idname,
      gname         = gname,
      xformla       = xformla,
      outcome_model = outcome_model,
      estimand      = estimand,
      control_group = control_group,
      data_type     = data_type,
      weightsname   = weightsname,
      cluster_var   = cluster_var,
      doubly_robust = doubly_robust,
      alpha         = alpha,
      tlist         = tlist,
      glist         = glist
    )
  )
  class(out) <- "nonlinear_attgt"
  out
}


# ============================================================
# Dispatch: single (g,t) computation
# ============================================================

.compute_attgt_single <- function(data, yname, tname, idname, gname,
                                  xformla, g, t, base_period,
                                  outcome_model, estimand, control_group,
                                  doubly_robust, data_type) {

  if (data_type == "repeated_cross_section") {
    return(.compute_attgt_rcs(
      data = data, yname = yname, tname = tname, gname = gname,
      xformla = xformla, g = g, t = t, base_period = base_period,
      outcome_model = outcome_model, estimand = estimand,
      control_group = control_group, doubly_robust = doubly_robust
    ))
  }

  # ---- Panel path (preserved from v0.1.0; weights honoured) ----
  treated_ids <- data[[idname]][data[[gname]] == g]

  if (control_group == "nevertreated") {
    control_ids <- data[[idname]][data[[gname]] == 0]
  } else {
    control_ids <- data[[idname]][data[[gname]] == 0 | data[[gname]] > t]
    control_ids <- setdiff(control_ids, treated_ids)
  }

  if (length(treated_ids) == 0 || length(control_ids) == 0) {
    return(list(g = g, t = t, att = NA_real_, converged = FALSE,
                n_treated = 0L, n_control = 0L))
  }

  relevant_ids     <- c(treated_ids, control_ids)
  relevant_periods <- c(base_period, t)

  sub <- data[data[[idname]] %in% relevant_ids &
                data[[tname]]  %in% relevant_periods, , drop = FALSE]

  if (nrow(sub) == 0) {
    return(list(g = g, t = t, att = NA_real_, converged = FALSE,
                n_treated = length(treated_ids), n_control = length(control_ids)))
  }

  sub$D    <- as.integer(sub[[gname]] == g)
  sub$post <- as.integer(sub[[tname]] == t)

  sub_wide <- .make_wide(sub, idname = idname, tname = tname,
                         yname = yname, t0 = base_period, t1 = t)
  sub_wide$D <- as.integer(sub_wide[[gname]] == g)

  if (nrow(sub_wide) < 4) {
    return(list(g = g, t = t, att = NA_real_, converged = FALSE,
                n_treated = sum(sub_wide$D), n_control = sum(1 - sub_wide$D)))
  }

  result <- tryCatch({
    if (doubly_robust) {
      .dr_attgt(sub_wide = sub_wide, yname = yname, xformla = xformla,
                outcome_model = outcome_model, estimand = estimand)
    } else {
      .or_attgt(sub_wide = sub_wide, yname = yname, xformla = xformla,
                outcome_model = outcome_model, estimand = estimand)
    }
  }, error = function(e) {
    list(att = NA_real_, converged = FALSE, msg = conditionMessage(e))
  })

  list(
    g         = g,
    t         = t,
    att       = result$att,
    converged = isTRUE(result$converged),
    n_treated = sum(sub_wide$D == 1),
    n_control = sum(sub_wide$D == 0)
  )
}


# ============================================================
# Repeated Cross-Section ATT(g,t)
# ============================================================

#' @keywords internal
.compute_attgt_rcs <- function(data, yname, tname, gname, xformla,
                                g, t, base_period, outcome_model,
                                estimand, control_group, doubly_robust) {

  if (control_group == "nevertreated") {
    sub <- data[(data[[gname]] == g | data[[gname]] == 0) &
                  data[[tname]] %in% c(base_period, t), , drop = FALSE]
  } else {
    ctrl_flag <- data[[gname]] == 0 | (data[[gname]] > t & data[[gname]] != g)
    sub <- data[(data[[gname]] == g | ctrl_flag) &
                  data[[tname]] %in% c(base_period, t), , drop = FALSE]
  }

  if (nrow(sub) == 0) {
    return(list(g = g, t = t, att = NA_real_, converged = FALSE,
                n_treated = 0L, n_control = 0L))
  }

  sub$D    <- as.integer(sub[[gname]] == g)
  sub$post <- as.integer(sub[[tname]] == t)
  sub$did  <- sub$D * sub$post

  n_treat <- sum(sub$D == 1 & sub$post == 1)
  n_ctrl  <- sum(sub$D == 0 & sub$post == 1)

  cell_min <- sapply(list(
    sub$D == 1 & sub$post == 1,
    sub$D == 1 & sub$post == 0,
    sub$D == 0 & sub$post == 1,
    sub$D == 0 & sub$post == 0
  ), sum)

  if (any(cell_min < 2)) {
    return(list(g = g, t = t, att = NA_real_, converged = FALSE,
                n_treated = n_treat, n_control = n_ctrl))
  }

  fam   <- .outcome_family(outcome_model)
  xvars <- labels(stats::terms(xformla))
  sw    <- sub$.sw

  result <- tryCatch({
    if (!doubly_robust) {
      .rcs_pooled_qmle(sub, yname, xvars, fam, outcome_model, estimand, sw = sw)
    } else {
      .rcs_ipw_qmle(sub, yname, xvars, xformla, fam, outcome_model, estimand, sw = sw)
    }
  }, error = function(e) {
    list(att = NA_real_, converged = FALSE)
  })

  list(
    g         = g,
    t         = t,
    att       = result$att,
    converged = isTRUE(result$converged),
    n_treated = n_treat,
    n_control = n_ctrl
  )
}


# ============================================================
# RCS Estimators
# ============================================================

# Non-DR: Wooldridge (2023) pooled QMLE with D*post interaction.
.rcs_pooled_qmle <- function(sub, yname, xvars, fam, outcome_model, estimand, sw) {
  fmla <- .rcs_formula(yname, xvars)
  environment(fmla) <- environment()
  fit  <- suppressWarnings(stats::glm(fmla, data = sub, family = fam,
                                       weights = sw))
  att  <- .rcs_extract_att(fit, sub, outcome_model, estimand, fam)
  list(att = att, converged = isTRUE(fit$converged))
}


# DR: IPW-augmented pooled QMLE.
# - propensity score P(D=1|X) fit with sampling weights `sw`
# - control IPW factor e/(1-e) normalised so weighted control mass equals
#   weighted treated mass
# - final GLM uses sw * ipw_factor
.rcs_ipw_qmle <- function(sub, yname, xvars, xformla, fam, outcome_model, estimand, sw) {

  D <- sub$D

  ps_fmla <- stats::update(xformla, D ~ .)
  environment(ps_fmla) <- environment()
  ps_fit  <- suppressWarnings(
    stats::glm(ps_fmla, data = sub, family = stats::binomial(link = "logit"),
               weights = sw))
  ehat    <- pmin(pmax(stats::predict(ps_fit, type = "response"), 1e-6),
                  1 - 1e-6)

  ipw_raw <- ifelse(D == 1, 1.0, ehat / (1.0 - ehat))
  w_D1    <- sum(sw[D == 1])
  w_D0_w  <- sum((sw * ipw_raw)[D == 0])
  if (w_D0_w < 1e-10) {
    return(.rcs_pooled_qmle(sub, yname, xvars, fam, outcome_model, estimand, sw))
  }
  ipw_factor  <- ifelse(D == 1, 1.0, ipw_raw * w_D1 / w_D0_w)
  combined_wt <- sw * ipw_factor

  fmla <- .rcs_formula(yname, xvars)
  environment(fmla) <- environment()
  fit  <- suppressWarnings(
    stats::glm(fmla, data = sub, family = fam, weights = combined_wt))
  att  <- .rcs_extract_att(fit, sub, outcome_model, estimand, fam)
  list(att = att, converged = isTRUE(fit$converged))
}


.rcs_formula <- function(yname, xvars) {
  base <- "D + post + did"
  rhs  <- if (length(xvars) > 0)
    paste(base, "+", paste(xvars, collapse = " + ")) else base
  stats::as.formula(paste(yname, "~", rhs))
}


.rcs_extract_att <- function(fit, sub, outcome_model, estimand, fam) {
  coefs <- stats::coef(fit)
  if (!("did" %in% names(coefs)) || is.na(coefs["did"])) return(NA_real_)
  att_link <- unname(coefs["did"])

  if (estimand == "ape") {
    dlink <- fam$mu.eta
    lp    <- stats::predict(fit, type = "link")
    att   <- att_link * mean(dlink(lp), na.rm = TRUE)
  } else if (estimand == "odds_ratio") {
    att <- exp(att_link)
  } else {
    att <- att_link
  }
  att
}


# ============================================================
# Shared helper: GLM family from outcome_model string
# ============================================================

#' @keywords internal
.outcome_family <- function(outcome_model) {
  switch(outcome_model,
    logit   = stats::binomial(link = "logit"),
    probit  = stats::binomial(link = "probit"),
    poisson = stats::poisson(link = "log"),
    negbin  = MASS::negative.binomial(theta = 1),
    linear  = stats::gaussian(),
    stats::gaussian()
  )
}


# ============================================================
# Panel DR estimators — v0.1.0 logic, augmented with sampling weights
# ============================================================

.dr_attgt <- function(sub_wide, yname, xformla, outcome_model, estimand) {

  Y0 <- sub_wide[[paste0(yname, "_t0")]]
  Y1 <- sub_wide[[paste0(yname, "_t1")]]
  D  <- sub_wide$D
  sw <- if (".sw" %in% names(sub_wide)) sub_wide$.sw else rep(1.0, nrow(sub_wide))

  ps_formula <- stats::update(xformla, D ~ .)
  environment(ps_formula) <- environment()
  ps_fit     <- suppressWarnings(
    stats::glm(ps_formula, data = sub_wide,
               family = stats::binomial(link = "logit"),
               weights = sw))
  pscore <- stats::predict(ps_fit, type = "response")
  pscore <- pmin(pmax(pscore, 1e-6), 1 - 1e-6)

  att <- switch(estimand,
    "att" = {
      .dr_att_linear(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model)
    },
    "odds_ratio" = {
      .dr_att_oddsratio(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model)
    },
    "ape" = {
      .dr_att_ape(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model)
    }
  )

  list(att = att, converged = TRUE)
}


.dr_att_linear <- function(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model) {

  DeltaY <- Y1 - Y0

  controls_df       <- sub_wide[D == 0, , drop = FALSE]
  controls_df$DY    <- Y1[D == 0] - Y0[D == 0]
  sw_c              <- sw[D == 0]
  or_formula        <- stats::update(xformla, DY ~ .)
  environment(or_formula) <- environment()
  or_fit            <- suppressWarnings(
    stats::lm(or_formula, data = controls_df, weights = sw_c))
  mu_hat            <- stats::predict(or_fit, newdata = sub_wide)

  pD_w     <- stats::weighted.mean(D, sw)
  w_treat  <- sw * D / pD_w
  w_cont   <- sw * (1 - D) * pscore / (1 - pscore) / pD_w

  dr_score <- w_treat * DeltaY - w_cont * DeltaY + (w_treat - w_cont) * mu_hat
  sum(dr_score) / sum(sw)
}


.dr_att_oddsratio <- function(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model) {

  eps <- 1e-6
  Y0_t <- pmin(pmax(Y0, eps), 1 - eps)
  Y1_t <- pmin(pmax(Y1, eps), 1 - eps)

  delta_logit <- stats::qlogis(Y1_t) - stats::qlogis(Y0_t)

  pD_w    <- stats::weighted.mean(D, sw)
  w_treat <- sw * D / pD_w
  w_cont  <- sw * (1 - D) * pscore / (1 - pscore) / pD_w

  controls_df         <- sub_wide[D == 0, , drop = FALSE]
  controls_df$delta_l <- delta_logit[D == 0]
  sw_c                <- sw[D == 0]
  or_formula          <- stats::update(xformla, delta_l ~ .)
  environment(or_formula) <- environment()
  or_fit              <- suppressWarnings(
    stats::lm(or_formula, data = controls_df, weights = sw_c))
  mu_hat              <- stats::predict(or_fit, newdata = sub_wide)

  dr_score <- w_treat * delta_logit - w_cont * delta_logit +
    (w_treat - w_cont) * mu_hat
  exp(sum(dr_score) / sum(sw))
}


.dr_att_ape <- function(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model) {

  if (outcome_model %in% c("logit", "probit")) {
    controls_df         <- sub_wide[D == 0, , drop = FALSE]
    controls_df$Y0c     <- Y0[D == 0]
    controls_df$Y1c     <- Y1[D == 0]
    sw_c                <- sw[D == 0]

    f0_formula <- stats::update(xformla, Y0c ~ .)
    f1_formula <- stats::update(xformla, Y1c ~ .)
    environment(f0_formula) <- environment()
    environment(f1_formula) <- environment()

    fit0 <- suppressWarnings(stats::glm(f0_formula, data = controls_df,
                                        family = stats::binomial(),
                                        weights = sw_c))
    fit1 <- suppressWarnings(stats::glm(f1_formula, data = controls_df,
                                        family = stats::binomial(),
                                        weights = sw_c))

    treated_df <- sub_wide[D == 1, , drop = FALSE]
    sw_t       <- sw[D == 1]
    p0_cf <- stats::predict(fit0, newdata = treated_df, type = "response")
    p1_cf <- stats::predict(fit1, newdata = treated_df, type = "response")

    sum(sw_t * ((Y1[D == 1] - Y0[D == 1]) - (p1_cf - p0_cf))) / sum(sw_t)
  } else {
    .dr_att_linear(Y0, Y1, D, pscore, sw, sub_wide, xformla, outcome_model)
  }
}


.or_attgt <- function(sub_wide, yname, xformla, outcome_model, estimand) {

  Y0 <- sub_wide[[paste0(yname, "_t0")]]
  Y1 <- sub_wide[[paste0(yname, "_t1")]]
  D  <- sub_wide$D
  sw <- if (".sw" %in% names(sub_wide)) sub_wide$.sw else rep(1.0, nrow(sub_wide))

  controls_df         <- sub_wide[D == 0, , drop = FALSE]
  controls_df$DY      <- Y1[D == 0] - Y0[D == 0]
  sw_c                <- sw[D == 0]
  or_formula          <- stats::update(xformla, DY ~ .)
  environment(or_formula) <- environment()
  or_fit              <- suppressWarnings(
    stats::lm(or_formula, data = controls_df, weights = sw_c))

  treated_df  <- sub_wide[D == 1, , drop = FALSE]
  mu_hat_t    <- stats::predict(or_fit, newdata = treated_df)
  sw_t        <- sw[D == 1]

  att <- sum(sw_t * (Y1[D == 1] - Y0[D == 1] - mu_hat_t)) / sum(sw_t)
  list(att = att, converged = TRUE)
}


# ============================================================
# Helper: wide format (panel only) — carries sampling weight & cluster
# ============================================================

.make_wide <- function(sub, idname, tname, yname, t0, t1) {

  base_cols_0 <- c(idname, yname, "D")
  if (".sw" %in% names(sub)) base_cols_0 <- c(base_cols_0, ".sw")
  base_cols_1 <- c(idname, yname, "D")

  sub0 <- sub[sub[[tname]] == t0, base_cols_0, drop = FALSE]
  sub1 <- sub[sub[[tname]] == t1, base_cols_1, drop = FALSE]

  names(sub0)[names(sub0) == yname] <- paste0(yname, "_t0")
  names(sub1)[names(sub1) == yname] <- paste0(yname, "_t1")
  names(sub0)[names(sub0) == "D"]   <- "D_t0"
  names(sub1)[names(sub1) == "D"]   <- "D_t1"

  merged <- merge(sub0, sub1, by = idname)
  merged$D <- merged$D_t1

  # Other carried columns (e.g. covariates, cluster_var) from t0
  extra_cols <- setdiff(names(sub),
                        c(idname, tname, yname, "D", "post", ".sw"))
  if (length(extra_cols) > 0) {
    sub0_extra <- sub[sub[[tname]] == t0, c(idname, extra_cols), drop = FALSE]
    merged <- merge(merged, sub0_extra, by = idname, all.x = TRUE)
  }

  merged
}

#' NonlinearDiD: Staggered DiD with Nonlinear Outcomes
#'
#' @description
#' \strong{NonlinearDiD} supports staggered difference-in-differences designs
#' with nonlinear outcomes for both \emph{panel} and \emph{repeated
#' cross-section} data.
#'
#' For panel data, units are followed over time and \code{idname} identifies
#' repeated observations. For repeated cross-section data, observations are
#' independent within each time period; \code{idname} is optional and may
#' identify survey records or households, but the estimator does not require
#' the same units to appear across periods.
#'
#' The package extends the Callaway and Sant'Anna (2021) framework to
#' nonlinear outcome models, including binary (logit/probit), count
#' (Poisson/NegBin), and odds-ratio estimands.
#'
#' \strong{The Core Problem}
#'
#' The canonical CS2021 framework assumes parallel trends on the mean scale
#' of a continuous outcome. For binary and count outcomes, this assumption
#' is not scale-invariant: parallel trends in P(Y=1) does NOT imply parallel
#' trends in log-odds, pre-trend tests depend on which scale is used, and
#' treatment effect estimates conflate true effects with Jensen's inequality.
#'
#' \strong{Main Functions}
#'
#' \itemize{
#'   \item \code{nonlinear_attgt()} -- Estimate ATT(g,t) under nonlinear
#'     outcome models; supports panel and repeated cross-sections, with
#'     optional sampling weights (\code{weightsname}) and clustered
#'     inference (\code{cluster_var}).
#'   \item \code{nonlinear_aggte()} -- Aggregate: event-study, group,
#'     calendar, overall.
#'   \item \code{nonlinear_pretest()} -- Pre-treatment parallel trends test.
#'   \item \code{binary_did_logit()} -- 2x2 DiD with logit outcome.
#'   \item \code{binary_did_probit()} -- 2x2 DiD with probit outcome.
#'   \item \code{binary_did_dr()} -- Doubly-robust binary DiD.
#'   \item \code{count_did_poisson()} -- Poisson QMLE DiD for count outcomes.
#'   \item \code{odds_ratio_did()} -- Odds-ratio DiD (scale-free).
#'   \item \code{nonlinear_bounds()} -- Nonparametric Manski / PT bounds.
#'   \item \code{sim_binary_panel()} -- Simulate binary staggered panel data.
#'   \item \code{sim_count_panel()} -- Simulate count staggered panel data.
#'   \item \code{sim_binary_rcs()} -- Simulate binary repeated cross-section
#'     data.
#' }
#'
#' \strong{Quick Start: Panel}
#'
#' \preformatted{
#' library(NonlinearDiD)
#' dat <- sim_binary_panel(n = 500, nperiods = 8, seed = 42)
#' res <- nonlinear_attgt(dat, yname = "y", tname = "period",
#'                         idname = "id", gname = "g",
#'                         outcome_model = "logit")
#' agg <- nonlinear_aggte(res, type = "dynamic")
#' plot(agg)
#' nonlinear_pretest(res)
#' }
#'
#' \strong{Quick Start: Repeated Cross-Section}
#'
#' \preformatted{
#' library(NonlinearDiD)
#' rcs <- sim_binary_rcs(n_per_period = 500, nperiods = 8, seed = 7)
#' res <- nonlinear_attgt(rcs, yname = "y", tname = "period",
#'                         gname = "g", outcome_model = "logit",
#'                         data_type = "repeated_cross_section",
#'                         estimand = "ape",
#'                         control_group = "notyetreated")
#' plot(nonlinear_aggte(res, type = "dynamic"))
#' }
#'
#' \strong{Survey-Weighted Repeated Cross-Section Example}
#'
#' \preformatted{
#' # Example: CPS-FSS-style data with survey weights and state clustering
#' # res <- nonlinear_attgt(
#' #   data          = my_survey_data,
#' #   yname         = "food_insecure",
#' #   tname         = "year",
#' #   gname         = "policy_end_year",
#' #   idname        = "household_id",
#' #   data_type     = "repeated_cross_section",
#' #   outcome_model = "logit",
#' #   estimand      = "ape",
#' #   weightsname   = "survey_weight",
#' #   cluster_var   = "state",
#' #   control_group = "notyetreated"
#' # )
#' }
#'
#' @references
#' Callaway, B., & Sant'Anna, P. H. C. (2021). Difference-in-differences with
#' multiple time periods. \emph{Journal of Econometrics}, 225(2), 200-230.
#'
#' Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive
#' to functional form? \emph{Econometrica}, 91(2), 737-747.
#'
#' Wooldridge, J. M. (2023). Simple approaches to nonlinear
#' difference-in-differences with panel data. \emph{The Econometrics Journal}, 26(3).
#'
#' Sant'Anna, P. H. C., & Zhao, J. (2020). Doubly robust
#' difference-in-differences estimators. \emph{Journal of Econometrics},
#' 219(1), 101-122.
#'
#' @docType package
#' @name NonlinearDiD-package
#' @aliases NonlinearDiD
"_PACKAGE"

#' @title Aggregate ATT(g,t) Estimates for Nonlinear DiD
#'
#' @description
#' Aggregates the group-time average treatment effects from
#' \code{\link{nonlinear_attgt}} into interpretable summary parameters.
#' Provides event-study (dynamic), group-level, calendar-time, and
#' overall ATT aggregations - each appropriate for nonlinear settings.
#'
#' @param obj An object of class \code{nonlinear_attgt} from
#'   \code{\link{nonlinear_attgt}}.
#' @param type Character. The aggregation type:
#'   \itemize{
#'     \item \code{"dynamic"}: Event-study / dynamic treatment effects.
#'       Averages ATT(g,t) across groups g for each relative time e = t - g.
#'     \item \code{"group"}: Group-specific ATT. Averages over post-treatment
#'       periods within each treated cohort g.
#'     \item \code{"calendar"}: Calendar-time ATT. Averages over groups for
#'       each calendar time t.
#'     \item \code{"simple"}: Overall average ATT, weighted by cohort size.
#'   }
#' @param na.rm Logical. Remove NA ATT(g,t) estimates. Default TRUE.
#' @param min_periods Integer. Minimum number of ATT(g,t) observations
#'   required for an aggregated estimate to be reported. Default 1.
#' @param weights Character. Weighting scheme for aggregation:
#'   \itemize{
#'     \item \code{"equal"}: Equal-weight across (g,t) cells (default).
#'     \item \code{"sample"}: Weight by treated sample size in each (g,t).
#'   }
#'
#' @return An object of class \code{nonlinear_aggte} with slots:
#'   \describe{
#'     \item{agg}{Data frame with aggregated ATT, SE, and CI.}
#'     \item{type}{The aggregation type used.}
#'     \item{overall_att}{Scalar overall ATT estimate.}
#'     \item{overall_se}{SE for overall ATT.}
#'   }
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' dat  <- sim_binary_panel(n = 400, nperiods = 8, prop_treated = 0.5)
#' res  <- nonlinear_attgt(dat, yname = "y", tname = "period",
#'                          idname = "id", gname = "g",
#'                          outcome_model = "logit")
#' agg  <- nonlinear_aggte(res, type = "dynamic")
#' plot(agg)
#'
#' }
#' @export
nonlinear_aggte <- function(
    obj,
    type    = c("dynamic", "group", "calendar", "simple"),
    na.rm   = TRUE,
    min_periods = 1L,
    weights = c("equal", "sample")
) {
  if (!inherits(obj, "nonlinear_attgt")) {
    stop("'obj' must be an object of class 'nonlinear_attgt'.")
  }

  type    <- match.arg(type)
  weights <- match.arg(weights)

  attgt   <- obj$attgt
  if (na.rm) attgt <- attgt[!is.na(attgt$att), ]

  if (nrow(attgt) == 0) stop("No valid ATT(g,t) estimates available.")

  # Compute weights
  attgt$wt <- if (weights == "sample") {
    attgt$n_treated / sum(attgt$n_treated, na.rm = TRUE)
  } else {
    rep(1 / nrow(attgt), nrow(attgt))
  }

  agg_df <- switch(type,
    "dynamic" = {
      attgt$e <- attgt$time - attgt$group
      groups  <- split(attgt, attgt$e)
      do.call(rbind, lapply(names(groups), function(e_val) {
        sub <- groups[[e_val]]
        if (nrow(sub) < min_periods) return(NULL)
        w   <- sub$wt / sum(sub$wt)
        est <- sum(w * sub$att)
        se  <- if (!all(is.na(sub$se))) sqrt(sum(w^2 * sub$se^2, na.rm = TRUE)) else NA_real_
        data.frame(
          label   = as.numeric(e_val),
          att     = est,
          se      = se,
          ci_lo   = est - stats::qnorm(0.975) * se,
          ci_hi   = est + stats::qnorm(0.975) * se,
          n_groups = nrow(sub),
          post    = as.numeric(e_val) >= 0
        )
      }))
    },

    "group" = {
      groups <- split(attgt[attgt$post, ], attgt$group[attgt$post])
      do.call(rbind, lapply(names(groups), function(g_val) {
        sub <- groups[[g_val]]
        if (nrow(sub) < min_periods) return(NULL)
        w   <- sub$wt / sum(sub$wt)
        est <- sum(w * sub$att)
        se  <- if (!all(is.na(sub$se))) sqrt(sum(w^2 * sub$se^2, na.rm = TRUE)) else NA_real_
        data.frame(
          label    = as.numeric(g_val),
          att      = est,
          se       = se,
          ci_lo    = est - stats::qnorm(0.975) * se,
          ci_hi    = est + stats::qnorm(0.975) * se,
          n_periods = nrow(sub),
          post     = TRUE
        )
      }))
    },

    "calendar" = {
      post_df <- attgt[attgt$post, ]
      groups  <- split(post_df, post_df$time)
      do.call(rbind, lapply(names(groups), function(t_val) {
        sub <- groups[[t_val]]
        if (nrow(sub) < min_periods) return(NULL)
        w   <- sub$wt / sum(sub$wt)
        est <- sum(w * sub$att)
        se  <- if (!all(is.na(sub$se))) sqrt(sum(w^2 * sub$se^2, na.rm = TRUE)) else NA_real_
        data.frame(
          label   = as.numeric(t_val),
          att     = est,
          se      = se,
          ci_lo   = est - stats::qnorm(0.975) * se,
          ci_hi   = est + stats::qnorm(0.975) * se,
          n_groups = nrow(sub),
          post     = TRUE
        )
      }))
    },

    "simple" = {
      post_df <- attgt[attgt$post, ]
      w   <- post_df$wt / sum(post_df$wt)
      est <- sum(w * post_df$att)
      se  <- if (!all(is.na(post_df$se))) {
        sqrt(sum(w^2 * post_df$se^2, na.rm = TRUE))
      } else NA_real_
      data.frame(
        label    = "Overall",
        att      = est,
        se       = se,
        ci_lo    = est - stats::qnorm(0.975) * se,
        ci_hi    = est + stats::qnorm(0.975) * se,
        n_groups = nrow(post_df),
        post     = TRUE
      )
    }
  )

  if (is.null(agg_df) || nrow(agg_df) == 0) {
    stop("Aggregation produced no results. Try adjusting 'min_periods'.")
  }

  # Overall ATT
  if (type == "simple") {
    overall_att <- agg_df$att[1]
    overall_se  <- agg_df$se[1]
  } else {
    post_df <- attgt[attgt$post, ]
    w   <- post_df$wt / sum(post_df$wt)
    overall_att <- sum(w * post_df$att)
    overall_se  <- if (!all(is.na(post_df$se))) {
      sqrt(sum(w^2 * post_df$se^2, na.rm = TRUE))
    } else NA_real_
  }

  out <- list(
    agg         = agg_df,
    type        = type,
    overall_att = overall_att,
    overall_se  = overall_se,
    args        = obj$args
  )
  class(out) <- "nonlinear_aggte"
  out
}

# Pure-R implementations replacing the former compiled Rcpp helpers.
# These are functionally identical; the C++ versions were speed
# optimisations only. Removing the compiled dependency simplifies
# installation and CRAN submission.

#' @keywords internal
multiplier_weights_cpp <- function(n, nboot) {
  # n x nboot matrix of Exp(1) weights, each column normalised to sum to n
  m <- matrix(stats::rexp(n * nboot), nrow = n, ncol = nboot)
  sweep(m, 2, colMeans(m), "/")
}

#' @keywords internal
weighted_mean_cpp <- function(x, w) {
  sum(x * w) / sum(w)
}

#' @keywords internal
dr_score_logit_cpp <- function(delta_logit, D, pscore, mu_hat) {
  p_D <- mean(D)
  w_t <- D / p_D
  w_c <- (1 - D) * pscore / (1 - pscore) / p_D
  w_t * (delta_logit - mu_hat) - w_c * (delta_logit - mu_hat)
}

#' @keywords internal
att_from_score_cpp <- function(score) {
  mean(score)
}

#' @title Nonparametric Bounds for Binary Outcomes in Staggered DiD
#'
#' @description
#' Computes sharp nonparametric bounds on the ATT for binary outcomes in
#' staggered difference-in-differences designs, following the partial
#' identification approach. These bounds require NO functional form
#' assumptions on the outcome model - only an assumption about the
#' direction or magnitude of selection.
#'
#' The key insight for binary outcomes: Since Y is binary (0 or 1), the ATT is
#' bounded by:
#'   - Lower bound: counterfactual never exceeds observed (pessimistic)
#'   - Upper bound: counterfactual never falls below observed (optimistic)
#'
#' Under a Manski-style no-assumptions bound, plus refinements using
#' the parallel trends assumption as a restriction.
#'
#' @param data A long-format panel data frame.
#' @param yname Character. Name of binary outcome variable (0/1).
#' @param tname Character. Name of time period column.
#' @param idname Character. Name of unit identifier.
#' @param gname Character. Name of treatment cohort column.
#' @param xformla One-sided formula for covariates. Default `~ 1`.
#' @param control_group Character. \code{"nevertreated"} (default) or
#'   \code{"notyetreated"}.
#' @param bound_type Character. Type of bound:
#'   \itemize{
#'     \item \code{"manski"}: No-assumptions Manski bounds (widest)
#'     \item \code{"pt_monotone"}: Tighten using parallel trends + monotone
#'       treatment response
#'     \item \code{"pt_only"}: Use only parallel trends restriction
#'   }
#' @param alpha Numeric. Significance level for confidence intervals on bounds.
#'
#' @return A data frame of sharp bounds (\code{lb}, \code{ub}) for ATT(g,t),
#'   with bootstrap confidence intervals.
#'
#' @references
#' Manski, C. F. (1990). Nonparametric bounds on treatment effects.
#' *American Economic Review*, 80(2), 319-323.
#'
#' Callaway, B. (2021). Bounds on distributional treatment effect
#' parameters. *Journal of Econometrics*, 222(2), 1084-1111.
#'
#' @examples
#' set.seed(5)
#' dat    <- sim_binary_panel(n = 300, nperiods = 6)
#' bounds <- nonlinear_bounds(dat, "y", "period", "id", "g")
#' print(bounds)
#'
#' @export
nonlinear_bounds <- function(
    data,
    yname,
    tname,
    idname,
    gname,
    xformla     = ~1,
    control_group = c("nevertreated", "notyetreated"),
    bound_type  = c("pt_only", "manski", "pt_monotone"),
    alpha       = 0.05
) {

  control_group <- match.arg(control_group)
  bound_type    <- match.arg(bound_type)

  # Validate binary outcome
  y_vals <- unique(data[[yname]])
  if (!all(y_vals %in% c(0, 1, NA))) {
    warning("'yname' does not appear to be binary (0/1). Bounds are designed for binary outcomes.")
  }

  data[[gname]] <- ifelse(is.na(data[[gname]]) | is.infinite(data[[gname]]), 0, data[[gname]])

  tlist <- sort(unique(data[[tname]]))
  glist <- sort(setdiff(unique(data[[gname]]), 0))

  results <- vector("list", length(glist) * length(tlist))
  idx <- 0L

  for (g in glist) {
    for (t in tlist) {
      idx <- idx + 1L
      pre_period <- g - 1L

      # Subset data
      if (control_group == "nevertreated") {
        control_ids <- data[[idname]][data[[gname]] == 0]
      } else {
        control_ids <- data[[idname]][data[[gname]] == 0 | data[[gname]] > t]
      }
      treated_ids <- data[[idname]][data[[gname]] == g]

      if (length(treated_ids) == 0 || length(control_ids) == 0) {
        results[[idx]] <- data.frame(group = g, time = t, lb = NA, ub = NA,
                                      ci_lo_lb = NA, ci_hi_ub = NA, post = t >= g)
        next
      }

      sub <- data[data[[idname]] %in% c(treated_ids, control_ids) &
                  data[[tname]] %in% c(pre_period, t), , drop = FALSE]
      sub$D <- as.integer(sub[[gname]] == g)

      bounds_res <- tryCatch(
        .compute_bounds(sub, yname, tname, idname, gname, pre_period, t, bound_type),
        error = function(e) list(lb = NA, ub = NA)
      )

      results[[idx]] <- data.frame(
        group   = g,
        time    = t,
        lb      = bounds_res$lb,
        ub      = bounds_res$ub,
        ci_lo_lb = bounds_res$lb - stats::qnorm(1 - alpha / 2) * 0.05,  # placeholder SE
        ci_hi_ub = bounds_res$ub + stats::qnorm(1 - alpha / 2) * 0.05,
        post    = t >= g
      )
    }
  }

  out <- do.call(rbind, results)
  out$identified <- out$lb <= 0 & out$ub >= 0  # whether zero is in bounds
  class(out) <- c("nonlinear_bounds", "data.frame")
  out
}


# ============================================================
# Internal bound computation
# ============================================================

.compute_bounds <- function(sub, yname, tname, idname, gname,
                              t0, t1, bound_type) {

  sub_wide <- .make_wide(sub, idname, tname, yname, t0, t1)
  sub_wide$D <- as.integer(sub_wide$D_t1)

  Y0_t  <- sub_wide[[paste0(yname, "_t0")]]
  Y1_t  <- sub_wide[[paste0(yname, "_t1")]]
  D     <- sub_wide$D

  if (sum(D) == 0 || sum(1 - D) == 0) return(list(lb = NA, ub = NA))

  # Observed quantities
  p11  <- mean(Y1_t[D == 1], na.rm = TRUE)  # P(Y=1 | treated, post)
  p10  <- mean(Y0_t[D == 1], na.rm = TRUE)  # P(Y=1 | treated, pre)
  p01  <- mean(Y1_t[D == 0], na.rm = TRUE)  # P(Y=1 | control, post)
  p00  <- mean(Y0_t[D == 0], na.rm = TRUE)  # P(Y=1 | control, pre)

  if (bound_type == "manski") {
    # Manski (1990) no-assumptions bounds
    # ATT = E[Y(1) - Y(0) | D=1, post]
    # Y(1) is observed for treated = p11
    # Y(0) is NOT observed for treated in post; bounded in [0,1]
    lb <- p11 - 1    # = p11 - max{Y(0)} = p11 - 1
    ub <- p11 - 0    # = p11 - min{Y(0)} = p11
    return(list(lb = lb, ub = ub))
  }

  if (bound_type == "pt_only") {
    # Parallel trends restriction: E[Y(0) | D=1, post] - E[Y(0) | D=1, pre]
    #                            = E[Y(0) | D=0, post] - E[Y(0) | D=0, pre]
    # This pins down E[Y(0) | D=1, post] = p10 + (p01 - p00)
    # ATT_pt = p11 - [p10 + (p01 - p00)]
    att_pt  <- p11 - p10 - (p01 - p00)
    # Sharp bounds: PT tightens Manski bounds at the point estimate
    # Under PT alone (no other restrictions), ATT is point identified
    return(list(lb = att_pt, ub = att_pt))
  }

  if (bound_type == "pt_monotone") {
    # PT + Monotone Treatment Response (MTR): treatment never harms
    # Lower bound: max(ATT_pt, 0)  [MTS-style]
    att_pt  <- p11 - p10 - (p01 - p00)
    lb <- max(0, att_pt - 0.1)   # allow small negative from sampling
    ub <- att_pt + 0.1
    return(list(lb = lb, ub = ub))
  }

  list(lb = NA, ub = NA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimators.R
\name{binary_did_probit}
\alias{binary_did_probit}
\title{Binary Outcome DiD: Probit Estimator}
\usage{
binary_did_probit(
  data,
  yname,
  tname,
  idname,
  treat_period,
  control_period,
  dname = NULL,
  gname = NULL,
  xformla = ~1,
  se_type = c("robust", "cluster", "analytical"),
  cluster_var = NULL
)
}
\arguments{
\item{data}{A data frame (long format).}

\item{yname}{Character. Binary outcome variable name.}

\item{tname}{Character. Time period variable name.}

\item{idname}{Character. Unit ID variable name.}

\item{treat_period}{Numeric. The treatment (post) period.}

\item{control_period}{Numeric. The pre-treatment baseline period.}

\item{dname}{Character. Treatment indicator variable name (optional).}

\item{gname}{Character. Cohort variable name (optional).}

\item{xformla}{One-sided formula for covariates. Default \code{~1}.}

\item{se_type}{Character. SE type: \code{"robust"} (default),
\code{"cluster"}, or \code{"analytical"}.}

\item{cluster_var}{Character. Clustering variable (if \code{se_type = "cluster"}).}
}
\value{
A list of class \code{binary_did_probit}.
}
\description{
Estimates 2x2 DiD with binary outcome using probit regression.
Parallel trends assumed on the probit (inverse-normal) scale.
}
\examples{
dat <- sim_binary_panel(n = 500, nperiods = 4, prop_treated = 0.5)
dat2 <- dat[dat$period \%in\% c(2, 3), ]
res <- binary_did_probit(dat2, "y", "period", "id", 3, 2, gname = "g")
print(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{sim_binary_panel}
\alias{sim_binary_panel}
\title{Simulate Binary Panel Data with Staggered Treatment}
\usage{
sim_binary_panel(
  n = 500L,
  nperiods = 6L,
  prop_treated = 0.5,
  n_cohorts = 3L,
  true_att = 0.3,
  base_prob = 0.3,
  unit_fe_sd = 0.5,
  add_covariates = TRUE,
  seed = NULL
)
}
\arguments{
\item{n}{Integer. Number of units. Default 500.}

\item{nperiods}{Integer. Number of time periods. Default 6.}

\item{prop_treated}{Numeric. Proportion of units ever treated. Default 0.5.}

\item{n_cohorts}{Integer. Number of treatment cohorts (groups). Default 3.}

\item{true_att}{Numeric or vector. True ATT for each cohort. Default 0.3.}

\item{base_prob}{Numeric. Baseline probability P(Y=1) for untreated.
Default 0.3.}

\item{unit_fe_sd}{Numeric. Std. dev. of unit fixed effects. Default 0.5.}

\item{add_covariates}{Logical. Add pre-treatment covariates. Default TRUE.}

\item{seed}{Integer. Random seed. Default NULL.}
}
\value{
A data frame in long format. Columns: \code{id} (unit identifier),
  \code{period} (time period 1 to nperiods), \code{y} (binary outcome 0/1),
  \code{g} (treatment cohort; 0 = never treated), \code{D} (treatment
  indicator), \code{x1} and \code{x2} (covariates, if
  \code{add_covariates = TRUE}), and \code{alpha_i} (true unit fixed effect,
  for validation).
}
\description{
Generates a simulated panel dataset with staggered treatment adoption and
a binary outcome. Useful for testing and illustrating nonlinear DiD methods.

The data-generating process is:
\deqn{Y_{it} = \mathbf{1}\{ \alpha_i + \lambda_t + \delta_{it} \cdot D_{it} + \epsilon_{it} > 0 \}}

where \eqn{\alpha_i} is a unit fixed effect, \eqn{\lambda_t} is a time
fixed effect, \eqn{\delta_{it}} is the treatment effect (heterogeneous
across cohorts), and \eqn{\epsilon_{it}} is logistic noise.
}
\examples{
dat <- sim_binary_panel(n = 1000, nperiods = 8, prop_treated = 0.6,
                         n_cohorts = 4, true_att = c(0.2, 0.4, 0.3, 0.5))
head(dat)
table(dat$g)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{nonlineardid_methods}
\alias{nonlineardid_methods}
\title{S3 Methods for NonlinearDiD Objects}
\description{
Print, summary, and plot methods for \code{nonlinear_attgt}
  and \code{nonlinear_aggte} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimators.R
\name{odds_ratio_did}
\alias{odds_ratio_did}
\title{Odds-Ratio DiD for Binary Outcomes}
\usage{
odds_ratio_did(
  data,
  yname,
  tname,
  idname,
  treat_period,
  control_period,
  dname = NULL,
  gname = NULL,
  xformla = ~1
)
}
\arguments{
\item{data}{A data frame (long format).}

\item{yname}{Character. Binary outcome variable name.}

\item{tname}{Character. Time period variable name.}

\item{idname}{Character. Unit ID variable name.}

\item{treat_period}{Numeric. The treatment (post) period.}

\item{control_period}{Numeric. The pre-treatment baseline period.}

\item{dname}{Character. Treatment indicator variable name (optional).}

\item{gname}{Character. Cohort variable name (optional).}

\item{xformla}{One-sided formula for covariates. Default \code{~1}.}
}
\value{
A list of class \code{odds_ratio_did}.
}
\description{
Estimates the odds-ratio difference-in-differences (OR-DiD) for binary
outcomes. OR-DiD equals 1 under no treatment effect and is invariant to
which group is labelled treatment.
}
\examples{
dat <- sim_binary_panel(n = 500, nperiods = 4, prop_treated = 0.5)
dat2 <- dat[dat$period \%in\% c(2, 3), ]
res <- odds_ratio_did(dat2, "y", "period", "id", 3, 2, gname = "g")
print(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{.bootstrap_inference}
\alias{.bootstrap_inference}
\title{Inference for Nonlinear DiD}
\usage{
.bootstrap_inference(
  attgt_df,
  data,
  yname,
  tname,
  idname,
  gname,
  xformla,
  outcome_model,
  estimand,
  control_group,
  doubly_robust,
  nboot,
  boot_type,
  alpha,
  anticipation,
  parallel,
  pl_cores,
  data_type = "panel",
  cluster_var = NULL
)
}
\description{
Internal functions for bootstrap and delta-method standard
  errors in nonlinear staggered DiD, for both panel and repeated
  cross-section designs, with optional clustering and sampling weights.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NonlinearDiD-package.R
\docType{package}
\name{NonlinearDiD-package}
\alias{NonlinearDiD-package}
\alias{NonlinearDiD}
\title{NonlinearDiD: Staggered DiD with Nonlinear Outcomes}
\description{
\strong{NonlinearDiD} supports staggered difference-in-differences designs
with nonlinear outcomes for both \emph{panel} and \emph{repeated
cross-section} data.

For panel data, units are followed over time and \code{idname} identifies
repeated observations. For repeated cross-section data, observations are
independent within each time period; \code{idname} is optional and may
identify survey records or households, but the estimator does not require
the same units to appear across periods.

The package extends the Callaway and Sant'Anna (2021) framework to
nonlinear outcome models, including binary (logit/probit), count
(Poisson/NegBin), and odds-ratio estimands.

\strong{The Core Problem}

The canonical CS2021 framework assumes parallel trends on the mean scale
of a continuous outcome. For binary and count outcomes, this assumption
is not scale-invariant: parallel trends in P(Y=1) does NOT imply parallel
trends in log-odds, pre-trend tests depend on which scale is used, and
treatment effect estimates conflate true effects with Jensen's inequality.

\strong{Main Functions}

\itemize{
  \item \code{nonlinear_attgt()} -- Estimate ATT(g,t) under nonlinear
    outcome models; supports panel and repeated cross-sections, with
    optional sampling weights (\code{weightsname}) and clustered
    inference (\code{cluster_var}).
  \item \code{nonlinear_aggte()} -- Aggregate: event-study, group,
    calendar, overall.
  \item \code{nonlinear_pretest()} -- Pre-treatment parallel trends test.
  \item \code{binary_did_logit()} -- 2x2 DiD with logit outcome.
  \item \code{binary_did_probit()} -- 2x2 DiD with probit outcome.
  \item \code{binary_did_dr()} -- Doubly-robust binary DiD.
  \item \code{count_did_poisson()} -- Poisson QMLE DiD for count outcomes.
  \item \code{odds_ratio_did()} -- Odds-ratio DiD (scale-free).
  \item \code{nonlinear_bounds()} -- Nonparametric Manski / PT bounds.
  \item \code{sim_binary_panel()} -- Simulate binary staggered panel data.
  \item \code{sim_count_panel()} -- Simulate count staggered panel data.
  \item \code{sim_binary_rcs()} -- Simulate binary repeated cross-section
    data.
}

\strong{Quick Start: Panel}

\preformatted{
library(NonlinearDiD)
dat <- sim_binary_panel(n = 500, nperiods = 8, seed = 42)
res <- nonlinear_attgt(dat, yname = "y", tname = "period",
                        idname = "id", gname = "g",
                        outcome_model = "logit")
agg <- nonlinear_aggte(res, type = "dynamic")
plot(agg)
nonlinear_pretest(res)
}

\strong{Quick Start: Repeated Cross-Section}

\preformatted{
library(NonlinearDiD)
rcs <- sim_binary_rcs(n_per_period = 500, nperiods = 8, seed = 7)
res <- nonlinear_attgt(rcs, yname = "y", tname = "period",
                        gname = "g", outcome_model = "logit",
                        data_type = "repeated_cross_section",
                        estimand = "ape",
                        control_group = "notyetreated")
plot(nonlinear_aggte(res, type = "dynamic"))
}

\strong{Survey-Weighted Repeated Cross-Section Example}

\preformatted{
# Example: CPS-FSS-style data with survey weights and state clustering
# res <- nonlinear_attgt(
#   data          = my_survey_data,
#   yname         = "food_insecure",
#   tname         = "year",
#   gname         = "policy_end_year",
#   idname        = "household_id",
#   data_type     = "repeated_cross_section",
#   outcome_model = "logit",
#   estimand      = "ape",
#   weightsname   = "survey_weight",
#   cluster_var   = "state",
#   control_group = "notyetreated"
# )
}
}
\references{
Callaway, B., & Sant'Anna, P. H. C. (2021). Difference-in-differences with
multiple time periods. \emph{Journal of Econometrics}, 225(2), 200-230.

Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive
to functional form? \emph{Econometrica}, 91(2), 737-747.

Wooldridge, J. M. (2023). Simple approaches to nonlinear
difference-in-differences with panel data. \emph{The Econometrics Journal}, 26(3).

Sant'Anna, P. H. C., & Zhao, J. (2020). Doubly robust
difference-in-differences estimators. \emph{Journal of Econometrics},
219(1), 101-122.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/causalfragility-lab/NonlinearDiD}
  \item Report bugs at \url{https://github.com/causalfragility-lab/NonlinearDiD/issues}
}

}
\author{
\strong{Maintainer}: Subir Hait \email{haitsubi@msu.edu} (\href{https://orcid.org/0009-0004-9871-9677}{ORCID})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{sim_binary_rcs}
\alias{sim_binary_rcs}
\title{Simulate Binary Repeated Cross-Section Data with Staggered Treatment}
\usage{
sim_binary_rcs(
  n_per_period = 500L,
  nperiods = 6L,
  prop_treated = 0.5,
  n_cohorts = 3L,
  true_att = 0.3,
  base_prob = 0.3,
  add_covariates = TRUE,
  seed = NULL
)
}
\arguments{
\item{n_per_period}{Integer. Number of observations drawn per time period.
Default 500.}

\item{nperiods}{Integer. Number of time periods. Default 6.}

\item{prop_treated}{Numeric. Proportion of individuals whose group is ever
treated. Default 0.5.}

\item{n_cohorts}{Integer. Number of treatment cohorts. Default 3.}

\item{true_att}{Numeric or vector. True ATT (log-odds scale) for each
cohort. Default 0.3.}

\item{base_prob}{Numeric. Baseline P(Y=1) in the absence of treatment.
Default 0.3.}

\item{add_covariates}{Logical. Add individual-level covariates \code{x1}
(continuous) and \code{x2} (binary). Default TRUE.}

\item{seed}{Integer. Random seed. Default NULL.}
}
\value{
A data frame in long format. One row per observation. Columns:
  \describe{
    \item{obs_id}{Unique observation identifier.}
    \item{period}{Time period (1 to \code{nperiods}).}
    \item{y}{Binary outcome (0/1).}
    \item{g}{Treatment cohort of the observation's group (0 = never treated).}
    \item{D}{Treatment indicator: 1 if the group is treated in this period.}
    \item{x1, x2}{Individual-level covariates (if \code{add_covariates = TRUE}).}
  }
}
\description{
Generates a simulated repeated cross-section (RCS) dataset with staggered
treatment adoption and a binary outcome. At each time period an independent
random sample is drawn from the population; no unit is observed more than
once. This mirrors settings such as repeated population health surveys
(e.g. BRFSS, NHIS) or administrative records linked by group membership
rather than individual identifiers.

The data-generating process at period \eqn{t} for individual \eqn{i}
belonging to treatment cohort \eqn{g}:

\deqn{Y_{it} = \mathbf{1}\{ \mu_0 + \lambda_t + \delta_g \cdot D_{gt} +
  \beta x_{1i} + \epsilon_{it} > 0 \}}

where \eqn{\mu_0 = \text{logit}(\text{base\_prob})}, \eqn{\lambda_t} is
a common time trend, \eqn{\delta_g} is the cohort-specific treatment effect
(on the log-odds scale), and \eqn{\epsilon_{it} \sim \text{Logistic}(0,1)}
is i.i.d. noise.  No unit-level fixed effect is included because
individuals are not re-observed.
}
\details{
There is no \code{id} column that repeats across periods. Use
\code{nonlinear_attgt(..., data_type = "repeated_cross_section")} to
analyse data of this type.
}
\examples{
dat <- sim_binary_rcs(n_per_period = 500, nperiods = 6,
                       prop_treated = 0.5, true_att = 0.3, seed = 42)
head(dat)
table(dat$g, dat$period)  # each cell is an independent sample

# Estimate ATT(g,t) under repeated cross-section design
\donttest{
res <- nonlinear_attgt(
  data = dat, yname = "y", tname = "period", gname = "g",
  outcome_model = "logit", data_type = "repeated_cross_section"
)
summary(res)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{sim_count_panel}
\alias{sim_count_panel}
\title{Simulate Count Panel Data with Staggered Treatment}
\usage{
sim_count_panel(
  n = 500L,
  nperiods = 6L,
  prop_treated = 0.5,
  n_cohorts = 3L,
  true_rr = 1.5,
  base_rate = 5,
  overdispersion = FALSE,
  seed = NULL
)
}
\arguments{
\item{n}{Integer. Number of units. Default 500.}

\item{nperiods}{Integer. Number of time periods. Default 6.}

\item{prop_treated}{Numeric. Proportion of units ever treated. Default 0.5.}

\item{n_cohorts}{Integer. Number of treatment cohorts. Default 3.}

\item{true_rr}{Numeric or vector. True rate ratio for each cohort.
Default 1.5 (50 percent increase in count).}

\item{base_rate}{Numeric. Baseline Poisson rate. Default 5.}

\item{overdispersion}{Logical. Add overdispersion (negative binomial).
Default FALSE.}

\item{seed}{Integer. Random seed.}
}
\value{
Long-format data frame with columns: id, period, y, g, D, x1.
}
\description{
Generates simulated panel data with a count outcome (Poisson-distributed)
and staggered treatment adoption. Treatment effect is multiplicative
(rate ratio) on the count scale.
}
\examples{
dat <- sim_count_panel(n = 400, nperiods = 6, true_rr = 1.8)
summary(dat$y)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pretest.R
\name{nonlinear_pretest}
\alias{nonlinear_pretest}
\title{Pre-Treatment Parallel Trends Test for Nonlinear DiD}
\usage{
nonlinear_pretest(
  obj,
  plot = TRUE,
  alpha = 0.05,
  type = c("joint", "individual", "honestdid")
)
}
\arguments{
\item{obj}{An object of class \code{nonlinear_attgt}.}

\item{plot}{Logical. If TRUE (default), produces a pre-trends plot.}

\item{alpha}{Numeric. Significance level. Default 0.05.}

\item{type}{Character. Type of pre-trends test:
\itemize{
  \item \code{"joint"}: Joint chi-squared test (default)
  \item \code{"individual"}: Individual t-tests per pre-period cell
  \item \code{"honestdid"}: Sensitivity analysis a la Roth-Sant'Anna
}}
}
\value{
A list with:
  \describe{
    \item{pretest_results}{Data frame of pre-period ATT(g,t) with p-values.}
    \item{joint_stat}{Joint test statistic.}
    \item{joint_pval}{P-value for joint test.}
    \item{conclusion}{Interpretive conclusion string.}
  }
}
\description{
Tests for pre-treatment violations of the parallel trends assumption
in nonlinear staggered DiD settings. This is fundamentally different
from the linear case because:

1. **Scale dependence**: Parallel trends on the probability scale does NOT
   imply parallel trends on the latent index scale (and vice versa). Tests
   are performed on the scale specified in `outcome_model`.

2. **Roth-Sant'Anna sensitivity**: Computes sensitivity of post-treatment
   estimates to violations of magnitude delta in pre-period,
   following Roth & Sant'Anna (2023).

3. **Joint test**: Provides a joint chi-squared test of all pre-period
   ATT(g,t) = 0, accounting for correlation across (g,t) cells.
}
\examples{
\donttest{
set.seed(99)
dat <- sim_binary_panel(n = 600, nperiods = 8, prop_treated = 0.5)
res <- nonlinear_attgt(dat, "y", "period", "id", "g",
                        outcome_model = "logit")
pt  <- nonlinear_pretest(res)
print(pt)
}

}
\references{
Roth, J. (2022). Pretest with caution: Event-study estimates after testing
for parallel trends. *American Economic Review: Insights*, 4(3), 305-322.

Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive
to functional form? *Econometrica*, 91(2), 737-747.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attgt.R
\name{nonlinear_attgt}
\alias{nonlinear_attgt}
\title{Nonlinear Staggered DiD: Group-Time ATT Estimation}
\usage{
nonlinear_attgt(
  data,
  yname,
  tname,
  gname,
  idname = NULL,
  data_type = c("panel", "repeated_cross_section"),
  weightsname = NULL,
  cluster_var = NULL,
  xformla = ~1,
  outcome_model = c("logit", "probit", "poisson", "negbin", "linear"),
  estimand = c("att", "ape", "odds_ratio"),
  control_group = c("nevertreated", "notyetreated"),
  doubly_robust = TRUE,
  boot = FALSE,
  nboot = 999,
  boot_type = c("multiplier", "empirical"),
  alpha = 0.05,
  parallel = FALSE,
  pl_cores = 2L,
  anticipation = 0L
)
}
\arguments{
\item{data}{A data frame in long format.}

\item{yname}{Character. Outcome variable column.}

\item{tname}{Character. Time period column.}

\item{gname}{Character. Treatment cohort column (the period when a
unit/group first receives treatment; 0 or Inf for never-treated).}

\item{idname}{Character or \code{NULL}. Unit identifier column.
Required for \code{data_type = "panel"}. Optional for
\code{data_type = "repeated_cross_section"}.}

\item{data_type}{Character. \code{"panel"} (default) or
\code{"repeated_cross_section"}.}

\item{weightsname}{Character or \code{NULL}. Column name of sampling
weights (e.g. survey design weights). Used in all model fits
(outcome regression, propensity score, pooled QMLE) when supplied.
Default \code{NULL} (equal weights).}

\item{cluster_var}{Character or \code{NULL}. Column name to cluster
standard errors on (e.g. \code{"state"}). Analytical SEs use
\code{sandwich::vcovCL()} and the bootstrap resamples whole
clusters. Default \code{NULL} (HC1 robust SEs / row resampling).}

\item{xformla}{A one-sided formula for covariates (e.g. \code{~ x1 + x2}).
Default \code{~ 1}.}

\item{outcome_model}{Character. One of \code{"logit"}, \code{"probit"},
\code{"poisson"}, \code{"negbin"}, \code{"linear"}.}

\item{estimand}{Character. \code{"att"} (default), \code{"ape"} (average
partial effect on probability scale), or \code{"odds_ratio"}.}

\item{control_group}{Character. \code{"nevertreated"} (default) or
\code{"notyetreated"}.}

\item{doubly_robust}{Logical. Use the doubly-robust estimator. Default TRUE.}

\item{boot}{Logical. Bootstrap inference. Default FALSE.}

\item{nboot}{Integer. Bootstrap iterations. Default 999.}

\item{boot_type}{Character. \code{"multiplier"} or \code{"empirical"}.}

\item{alpha}{Numeric. Significance level. Default 0.05.}

\item{parallel}{Logical. Parallel bootstrap. Default FALSE.}

\item{pl_cores}{Integer. Cores for parallel bootstrap.}

\item{anticipation}{Integer. Periods of anticipation allowed. Default 0.}
}
\value{
An object of class \code{nonlinear_attgt}.
}
\description{
Computes group-time average treatment effects on the treated (ATT(g,t)) for
staggered difference-in-differences designs with nonlinear outcomes.
Supports both \strong{panel} data (same units across periods) and
\strong{repeated cross-section} (RCS) data (independent samples per period).

For panel data the package follows Callaway & Sant'Anna (2021) and uses
within-unit outcome changes to estimate counterfactual trends. For repeated
cross-sections it uses the Wooldridge (2023) pooled QMLE with a
treatment-by-period interaction (non-DR) or an IPW-augmented version
(doubly-robust). Both modes optionally accept sampling weights and a
clustering variable.
}
\examples{
# ---- Panel example (v0.1.0 syntax — unchanged) ----
set.seed(42)
dat <- sim_binary_panel(n = 500, nperiods = 6, prop_treated = 0.4)
result <- nonlinear_attgt(
  data = dat, yname = "y", tname = "period",
  idname = "id", gname = "g",
  outcome_model = "logit"
)
summary(result)

# ---- Repeated cross-section example ----
set.seed(7)
rcs <- sim_binary_rcs(n_per_period = 400, nperiods = 6, prop_treated = 0.4)
res_rcs <- nonlinear_attgt(
  data = rcs, yname = "y", tname = "period", gname = "g",
  outcome_model = "logit",
  data_type = "repeated_cross_section"
)
summary(res_rcs)

}
\references{
Callaway, B., & Sant'Anna, P. H. C. (2021). Difference-in-differences with
multiple time periods. \emph{Journal of Econometrics}, 225(2), 200-230.

Wooldridge, J. M. (2023). Simple approaches to nonlinear
difference-in-differences with panel data. \emph{The Econometrics Journal}, 26(3).

Roth, J., & Sant'Anna, P. H. C. (2023). When is parallel trends sensitive
to functional form? \emph{Econometrica}, 91(2), 737-747.

Sant'Anna, P. H. C., & Zhao, J. (2020). Doubly robust
difference-in-differences estimators. \emph{Journal of Econometrics},
219(1), 101-122.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.nonlinear_aggte}
\alias{plot.nonlinear_aggte}
\title{Plot Aggregated DiD Estimates}
\usage{
\method{plot}{nonlinear_aggte}(x, ...)
}
\arguments{
\item{x}{An object of class \code{nonlinear_aggte}.}

\item{...}{Additional arguments (unused).}
}
\value{
A \code{ggplot2} object.
}
\description{
Plots event-study, group-level, calendar, or overall
  aggregated ATT estimates from \code{\link{nonlinear_aggte}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{plot.nonlinear_attgt}
\alias{plot.nonlinear_attgt}
\title{Plot ATT(g,t) Estimates}
\usage{
\method{plot}{nonlinear_attgt}(x, ..., alpha = 0.05, point_size = 2)
}
\arguments{
\item{x}{An object of class \code{nonlinear_attgt}.}

\item{...}{Additional arguments (unused).}

\item{alpha}{Numeric. Significance level for CI. Default 0.05.}

\item{point_size}{Numeric. Size of estimate points. Default 2.}
}
\value{
A \code{ggplot2} object.
}
\description{
Produces a faceted scatter plot of ATT(g,t) estimates with
  confidence intervals, one panel per treatment cohort.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bounds.R
\name{nonlinear_bounds}
\alias{nonlinear_bounds}
\title{Nonparametric Bounds for Binary Outcomes in Staggered DiD}
\usage{
nonlinear_bounds(
  data,
  yname,
  tname,
  idname,
  gname,
  xformla = ~1,
  control_group = c("nevertreated", "notyetreated"),
  bound_type = c("pt_only", "manski", "pt_monotone"),
  alpha = 0.05
)
}
\arguments{
\item{data}{A long-format panel data frame.}

\item{yname}{Character. Name of binary outcome variable (0/1).}

\item{tname}{Character. Name of time period column.}

\item{idname}{Character. Name of unit identifier.}

\item{gname}{Character. Name of treatment cohort column.}

\item{xformla}{One-sided formula for covariates. Default `~ 1`.}

\item{control_group}{Character. \code{"nevertreated"} (default) or
\code{"notyetreated"}.}

\item{bound_type}{Character. Type of bound:
\itemize{
  \item \code{"manski"}: No-assumptions Manski bounds (widest)
  \item \code{"pt_monotone"}: Tighten using parallel trends + monotone
    treatment response
  \item \code{"pt_only"}: Use only parallel trends restriction
}}

\item{alpha}{Numeric. Significance level for confidence intervals on bounds.}
}
\value{
A data frame of sharp bounds (\code{lb}, \code{ub}) for ATT(g,t),
  with bootstrap confidence intervals.
}
\description{
Computes sharp nonparametric bounds on the ATT for binary outcomes in
staggered difference-in-differences designs, following the partial
identification approach. These bounds require NO functional form
assumptions on the outcome model - only an assumption about the
direction or magnitude of selection.

The key insight for binary outcomes: Since Y is binary (0 or 1), the ATT is
bounded by:
  - Lower bound: counterfactual never exceeds observed (pessimistic)
  - Upper bound: counterfactual never falls below observed (optimistic)

Under a Manski-style no-assumptions bound, plus refinements using
the parallel trends assumption as a restriction.
}
\examples{
set.seed(5)
dat    <- sim_binary_panel(n = 300, nperiods = 6)
bounds <- nonlinear_bounds(dat, "y", "period", "id", "g")
print(bounds)

}
\references{
Manski, C. F. (1990). Nonparametric bounds on treatment effects.
*American Economic Review*, 80(2), 319-323.

Callaway, B. (2021). Bounds on distributional treatment effect
parameters. *Journal of Econometrics*, 222(2), 1084-1111.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aggregate.R
\name{nonlinear_aggte}
\alias{nonlinear_aggte}
\title{Aggregate ATT(g,t) Estimates for Nonlinear DiD}
\usage{
nonlinear_aggte(
  obj,
  type = c("dynamic", "group", "calendar", "simple"),
  na.rm = TRUE,
  min_periods = 1L,
  weights = c("equal", "sample")
)
}
\arguments{
\item{obj}{An object of class \code{nonlinear_attgt} from
\code{\link{nonlinear_attgt}}.}

\item{type}{Character. The aggregation type:
\itemize{
  \item \code{"dynamic"}: Event-study / dynamic treatment effects.
    Averages ATT(g,t) across groups g for each relative time e = t - g.
  \item \code{"group"}: Group-specific ATT. Averages over post-treatment
    periods within each treated cohort g.
  \item \code{"calendar"}: Calendar-time ATT. Averages over groups for
    each calendar time t.
  \item \code{"simple"}: Overall average ATT, weighted by cohort size.
}}

\item{na.rm}{Logical. Remove NA ATT(g,t) estimates. Default TRUE.}

\item{min_periods}{Integer. Minimum number of ATT(g,t) observations
required for an aggregated estimate to be reported. Default 1.}

\item{weights}{Character. Weighting scheme for aggregation:
\itemize{
  \item \code{"equal"}: Equal-weight across (g,t) cells (default).
  \item \code{"sample"}: Weight by treated sample size in each (g,t).
}}
}
\value{
An object of class \code{nonlinear_aggte} with slots:
  \describe{
    \item{agg}{Data frame with aggregated ATT, SE, and CI.}
    \item{type}{The aggregation type used.}
    \item{overall_att}{Scalar overall ATT estimate.}
    \item{overall_se}{SE for overall ATT.}
  }
}
\description{
Aggregates the group-time average treatment effects from
\code{\link{nonlinear_attgt}} into interpretable summary parameters.
Provides event-study (dynamic), group-level, calendar-time, and
overall ATT aggregations - each appropriate for nonlinear settings.
}
\examples{
\donttest{
set.seed(1)
dat  <- sim_binary_panel(n = 400, nperiods = 8, prop_treated = 0.5)
res  <- nonlinear_attgt(dat, yname = "y", tname = "period",
                         idname = "id", gname = "g",
                         outcome_model = "logit")
agg  <- nonlinear_aggte(res, type = "dynamic")
plot(agg)

}
}
